package com.ninegag.android.blitz2.ui.delegate

import androidx.recyclerview.widget.LinearLayoutManager
import androidx.recyclerview.widget.RecyclerView
import timber.log.Timber

import android.widget.LinearLayout

import com.ninegag.android.blitz2.BlitzActionListener
import com.ninegag.android.blitz2.BlitzState
import com.ninegag.android.blitz2.BlitzView

/**
 * Created by raymond on 10/8/2016.
 */

class LinearPaginationDetector @JvmOverloads constructor(private val actionListener: BlitzActionListener,
                                                         loadMoreBound: Int = DEFAULT_LOAD_MORE_BOUND, shouldTriggeredOnAttachment: Boolean = true) : PaginationDetector {

    private var orientation = VERTICAL
    private var loadMoreBound = DEFAULT_LOAD_MORE_BOUND // default 20
    private var shouldTriggeredOnAttachment = true

    private var lastTriggerLoadNext: Long = 0
    private var lastTriggerLoadPrev: Long = 0

    init {
        this.loadMoreBound = loadMoreBound
        this.shouldTriggeredOnAttachment = shouldTriggeredOnAttachment
    }

    override fun checkPagination(blitzView: BlitzView, layoutManager: RecyclerView.LayoutManager?, dx: Int, dy: Int) {
        val linearLayoutManager = layoutManager as LinearLayoutManager?
        if (VERTICAL == orientation) {
            checkLoadVertical(blitzView, linearLayoutManager, dy)
        } else {
            checkLoadHorizontal(blitzView, linearLayoutManager, dx)
        }
    }

    /**
     * By default, [android.support.v7.widget.RecyclerView.OnScrollListener.onScrolled]
     * will be triggered after scroll listener is added, we use this to configure
     * if we want this behaviour
     */
    override fun shouldTriggerOnAttachment(): Boolean {
        return this.shouldTriggeredOnAttachment
    }

    private fun checkLoadVertical(blitzView: BlitzView, linearLayoutManager: LinearLayoutManager?, dy: Int) {

        val now = System.currentTimeMillis()
        val viewState = blitzView.viewState

        if (linearLayoutManager == null) return
        if (viewState == BlitzState.STATE_INIT_LOADING ||
                viewState == BlitzState.STATE_REFRESHING ||
                viewState == BlitzState.STATE_LOADING_MORE)
            return

        val itemCount = linearLayoutManager.itemCount
        val visibleItemCount = linearLayoutManager.childCount
        val firstVisibleItem = linearLayoutManager.findFirstVisibleItemPosition()

        Timber.d("checkLoadVertical: itemCount=" + itemCount +
                ", visibleItemCount=" + visibleItemCount +
                ", firstVisibleItem=" + firstVisibleItem)

        val offset = if (loadMoreBound <= itemCount) loadMoreBound else loadMoreBound / 2
        if (dy > 0 && firstVisibleItem + visibleItemCount + offset >= itemCount) {
            if (now - lastTriggerLoadNext < LOAD_MORE_CHECK_RATE_LIMIT) return
            if (viewState == BlitzState.STATE_FINISH_LOADING_NEXT_END_OF_LIST) return

            linearLayoutManager.stackFromEnd = false
            actionListener.onTriggerLoadNext()
            lastTriggerLoadNext = now

            Timber.d("checkLoadVertical: --loadNext=$lastTriggerLoadNext")
        }

        if (dy < 0 && firstVisibleItem == 0) {
            if (now - lastTriggerLoadPrev < LOAD_MORE_CHECK_RATE_LIMIT) return
            if (viewState == BlitzState.STATE_FINISH_LOADING_PREV_END_OF_LIST) return
            linearLayoutManager.stackFromEnd = true
            actionListener.onTriggerLoadPrevious()
            lastTriggerLoadPrev = now

            Timber.d("checkLoadVertical: --loadPrev=$lastTriggerLoadPrev")
        }
    }

    private fun checkLoadHorizontal(blitzView: BlitzView, linearLayoutManager: LinearLayoutManager?, dx: Int) {
        // TODO: 10/8/2016
    }

    fun setOrientation(orientation: Int) {
        this.orientation = orientation
    }

    companion object {

        private val TAG = "LinearPaginationDetecto"
        val VERTICAL = LinearLayout.VERTICAL
        val HORIZONTAL = LinearLayout.HORIZONTAL

        private val LOAD_MORE_CHECK_RATE_LIMIT: Long = 1000
        private val DEFAULT_LOAD_MORE_BOUND = 20
    }
}
