package com.under9.android.comments.api

import android.util.Log
import com.under9.android.comments.CommentSystem
import com.under9.android.comments.controller.DataController
import com.under9.android.comments.GsonFactory
import com.under9.android.lib.network.HttpClientFactory

import okhttp3.OkHttpClient
import retrofit2.Retrofit
import retrofit2.adapter.rxjava2.RxJava2CallAdapterFactory
import retrofit2.converter.gson.GsonConverterFactory

class CommentApi private constructor(private val commentSystem: CommentSystem,
                                     private val dataController: DataController) {

    private var client: OkHttpClient? = null

    val cdnApiService: CommentCdnApiService by lazy(LazyThreadSafetyMode.NONE) {
        provideRetrofitClient(getClient(), commentSystem.config.apiCdnPrefix)
                .create(CommentCdnApiService::class.java)
    }
    val apiService: CommentApiService by lazy(LazyThreadSafetyMode.NONE) {
        provideRetrofitClient(getClient(), commentSystem.config.apiPrefix)
                .create(CommentApiService::class.java)
    }

    private fun getClient(): OkHttpClient {
        if (client == null) {
            Log.d("CommentApi", "getClient, debug=" + commentSystem.isDebug)
            client = HttpClientFactory.createHttpClientBuilder(commentSystem.isDebug)
                    .addInterceptor {
                        chain ->
                        val originalRequest = chain.request()
                        val httpUrl = originalRequest.url()

                        originalRequest.method()

                        val newRequestBuilder = originalRequest.newBuilder()

                        newRequestBuilder.addHeader(HEADER_APP_ID, dataController.appId)
                                .addHeader(HEADER_USER_AGENT, commentSystem.config.userAgent)
                                .addHeader(HEADER_X_PACKAGE_ID, commentSystem.appId ?: "")
                                .addHeader(HEADER_X_DEVICE_UUID, commentSystem.config.deviceUUID)
                                .addHeader(HEADER_X_PACKAGE_VERSION, commentSystem.appVersion.toString())

                        if (commentSystem.authHash != null && !httpUrl.url().toString().contains(commentSystem.config.apiCdnPrefix)) {
                            newRequestBuilder.addHeader(HEADER_APP_AUTH, commentSystem.authHash)
                        }

                        chain.proceed(newRequestBuilder.build())
                    }
                    .build()
        }
        return client!!
    }

    private fun provideRetrofitClient(client: OkHttpClient?, baseUrl: String): Retrofit {
        val builder = Retrofit.Builder()
                .client(client!!)
                .baseUrl("${baseUrl}v1/")
                .addConverterFactory(GsonConverterFactory.create(GsonFactory.getStandardGson()))
                .addCallAdapterFactory(RxJava2CallAdapterFactory.create())

        return builder.build()
    }

    companion object {
        private const val HEADER_APP_ID = "appId"
        private const val HEADER_APP_AUTH = "auth"
        private const val HEADER_USER_AGENT = "User-Agent"
        private const val HEADER_X_PACKAGE_ID = "X-Package-ID"
        private const val HEADER_X_PACKAGE_VERSION = "X-Package-Version"
        private const val HEADER_X_DEVICE_UUID = "X-Device-UUID"

        @JvmStatic val instance: CommentApi by lazy {
            CommentApi(CommentSystem.getInstance(), DataController.getInstance())
        }
    }
}
