package com.under9.android.comments.controller

import com.under9.android.comments.CommentSystem
import com.under9.android.comments.api.CommentApi
import com.under9.android.comments.data.repository.AppInfoRepository
import com.under9.android.comments.data.repository.AppInfoRepositoryInterface
import com.under9.android.comments.data.repository.CacheableCommentListRepository
import com.under9.android.comments.data.repository.CommentListRepository
import com.under9.android.comments.data.repository.CommentListRepositoryInterface
import com.under9.android.comments.data.repository.UserRepository
import com.under9.android.comments.data.repository.UserRepositoryInterface
import com.under9.android.comments.GsonFactory

object RepositoryManager {
    private val dataController = DataController.getInstance()
    private val commentSystem = CommentSystem.getInstance()

    @JvmStatic
    val userRepository: UserRepositoryInterface by lazy {
        UserRepository(dataController, CommentApi.instance.apiService,
                GsonFactory.getStandardGson(), commentSystem)
    }

    @JvmStatic
    val appInfoRepository: AppInfoRepositoryInterface by lazy {
        AppInfoRepository(dataController, CommentApi.instance.cdnApiService)
    }

    @JvmStatic
    val commentListRepository: CommentListRepositoryInterface by lazy {
        CommentListRepository(dataController, CommentApi.instance.apiService)
    }

    @JvmStatic
    val cacheableCommentListRepository: CommentListRepositoryInterface by lazy {
        CacheableCommentListRepository(dataController, CommentApi.instance.cdnApiService)
    }

    /**
     * Determine if a cacheable version should be got
     * @param url
     * @return
     */
    @JvmStatic
    fun getCommentListRepositoryByUrl(url: String): CommentListRepositoryInterface {
        return if (userRepository.isPostCommented(url) || userRepository.isPostReported(url)) {
            commentListRepository
        } else {
            cacheableCommentListRepository
        }
    }
}
