package com.under9.android.comments.data.processor

import com.under9.android.comments.AppOptions
import com.under9.android.comments.controller.DataController
import com.under9.android.comments.data.query.info.AppInfoResult
import com.under9.android.comments.model.api.ApiInfo
import io.reactivex.Flowable

class AppInfoProcessor(dataController: DataController): BaseApiResponseProcessor<ApiInfo>(dataController) {

    override fun process(apiResponse: ApiInfo): Flowable<AppInfoResult> {
        val now = System.currentTimeMillis()

        apiResponse.payload?.let {
            dataController.run {
                setIntData(AppOptions.FIELD_MAX_MESSAGE_LENGTH, it.app.maxMessageLength)
                setIntData(AppOptions.FIELD_MAX_DISPLAY_LEVEL, it.app.maxDisplayLevel)
                setLongData(AppOptions.FIELD_LAST_INFO_UPDATE_TIME, now)

                setIntData(AppOptions.FIELD_ALLOW_ANONYMOUS_COMMENT, it.app.allowAnonymousComment)
                setIntData(AppOptions.FIELD_ENABLE_OP_LABEL_FEATURE, it.app.opLabelFeature)
                setStringData(AppOptions.FIELD_ANONYMOUS_AVATAR_URL, it.app.anonymousAvatarUrl)
                setIntData(AppOptions.FIELD_USER_NAME_CLICKABLE, it.app.mobileUserNameClickable)

                setIntData(AppOptions.FIELD_ALLOW_IMAGE_UPLOAD_FEATURE, it.app.allowImageUploadFeature)
                setIntData(AppOptions.FIELD_ALLOW_ANONYMOUS_VOTE, it.app.allowAnonymousVote)
                setIntData(AppOptions.FIELD_ENABLE_OP_HIDE_COMMENT_FEATURE, it.app.opHideCommentFeature)

                return Flowable.just(it.app.run {
                    AppInfoResult(
                            maxMessageLength = maxMessageLength,
                            maxDisplayLevel = maxDisplayLevel,
                            lastInfoUpdateTime = now,
                            allowAnonymousComment = allowAnonymousComment,
                            opLabelFeature = opLabelFeature,
                            anonymousAvatarUrl = anonymousAvatarUrl,
                            mobileUserNameClickable = mobileUserNameClickable,
                            allowImageUploadFeature = allowImageUploadFeature,
                            allowAnonymousVote = allowAnonymousVote,
                            opHideCommentFeature = opHideCommentFeature)
                })
            }
        }

        return Flowable.just(AppInfoResult.getDefaultOrCached(dataController))
    }
}