package com.under9.android.comments.data.processor

import androidx.collection.ArrayMap
import com.google.gson.reflect.TypeToken
import com.under9.android.comments.AppOptions
import com.under9.android.comments.controller.DataController
import com.under9.android.comments.data.query.user.UserStatusQueryResult
import com.under9.android.comments.model.api.ApiUserStatus
import com.under9.android.comments.GsonFactory
import io.reactivex.Flowable

class UserStatusProcessor(dataController: DataController): BaseApiResponseProcessor<ApiUserStatus>(dataController) {

    override fun process(apiResponse: ApiUserStatus): Flowable<UserStatusQueryResult> {
        apiResponse.payload?.let {
            it.user?.takeIf { it.userId != null }?.let {
                val user = dataController.obtainUser(it)
                dataController.setStringData(AppOptions.FIELD_LOGGED_IN_USER, user.userId)
            }

            it.quota?.let {
                dataController.setCommentQuota(it.count, it.wait)
            }

            it.likeMapping?.let {
                val likeMapping = GsonFactory.getStandardGson().toJson(it)
                dataController.setStringData(AppOptions.FIELD_LIKE_MAPPING, likeMapping)
            }

            it.reportedPostUrls?.let {
                val reportedPostUrls = GsonFactory.getStandardGson().toJson(it)
                dataController.setStringData(AppOptions.FIELD_REPORTED_POST_URLS, reportedPostUrls)
            }

            it.commentedPostUrls?.let {
                val commentedPostUrls = GsonFactory.getStandardGson().toJson(it)
                dataController.setStringData(AppOptions.FIELD_COMMENTED_POST_LIKE_URLS, commentedPostUrls)
            }

            dataController.setLongData(AppOptions.FIELD_LAST_USER_STATUS_QUERY_TIME, System.currentTimeMillis())

            return Flowable.just(UserStatusQueryResult(it.likeMapping, it.commentedPostUrls, it.reportedPostUrls))
        }

        val gson = GsonFactory.getStandardGson()
        val mapType = object: TypeToken<ArrayMap<String, Int>>() {}.type

        val cachedLikeMapping = gson.fromJson<Map<String, Int>>(dataController.getStringData(AppOptions.FIELD_LIKE_MAPPING), mapType)
        val cachedReportedPostUrls = gson.fromJson<Map<String, Int>>(dataController.getStringData(AppOptions.FIELD_REPORTED_POST_URLS), mapType)
        val cachedCommentedPostUrls = gson.fromJson<Map<String, Int>>(dataController.getStringData(AppOptions.FIELD_COMMENTED_POST_LIKE_URLS), mapType)

        return Flowable.just(UserStatusQueryResult(cachedLikeMapping, cachedCommentedPostUrls, cachedReportedPostUrls))
    }

}