package com.under9.android.comments.data.query.info

import com.under9.android.comments.AbstractCommentSystemConfig
import com.under9.android.comments.AppOptions
import com.under9.android.comments.controller.DataController

data class AppInfoResult(
        val maxMessageLength: Int,
        val maxDisplayLevel: Int,
        val lastInfoUpdateTime: Long,
        val allowAnonymousComment: Int,
        val opLabelFeature: Int,
        val anonymousAvatarUrl: String?,
        val mobileUserNameClickable: Int,
        val allowImageUploadFeature: Int,
        val allowAnonymousVote: Int,
        val opHideCommentFeature: Int
) {
    companion object {
        /**
         * Get the default or already cached settings
         */
        @JvmStatic fun getDefaultOrCached(dataController: DataController): AppInfoResult {
            return dataController.run {
                AppInfoResult(
                        maxMessageLength = getIntData(AppOptions.FIELD_MAX_MESSAGE_LENGTH, AbstractCommentSystemConfig.DEFAULT_TEXT_LIMIT),
                        maxDisplayLevel = getIntData(AppOptions.FIELD_MAX_DISPLAY_LEVEL, AbstractCommentSystemConfig.DEFAULT_MAX_DISPLAY_LEVEL),
                        lastInfoUpdateTime = getLongData(AppOptions.FIELD_LAST_INFO_UPDATE_TIME, System.currentTimeMillis()),
                        allowAnonymousComment = getIntData(AppOptions.FIELD_ALLOW_ANONYMOUS_COMMENT, AbstractCommentSystemConfig.DEFAULT_ALLOW_ANONYMOUS_COMMENT),
                        opLabelFeature = getIntData(AppOptions.FIELD_ENABLE_OP_LABEL_FEATURE, AbstractCommentSystemConfig.DEFAULT_ENABLE_OP_LABEL_FEATURE),
                        anonymousAvatarUrl = getStringData(AppOptions.FIELD_ANONYMOUS_AVATAR_URL),
                        mobileUserNameClickable = getIntData(AppOptions.FIELD_USER_NAME_CLICKABLE, 0),
                        allowImageUploadFeature = getIntData(AppOptions.FIELD_ALLOW_IMAGE_UPLOAD_FEATURE, AbstractCommentSystemConfig.DEFAULT_ALLOW_IMAGE_UPLOAD_FEATURE),
                        allowAnonymousVote = getIntData(AppOptions.FIELD_ALLOW_ANONYMOUS_VOTE, 0),
                        opHideCommentFeature = getIntData(AppOptions.FIELD_ENABLE_OP_HIDE_COMMENT_FEATURE, AbstractCommentSystemConfig.DEFAULT_ENABLE_OP_HIDE_COMMENT_FEATURE)
                )
            }
        }
    }
}