package com.under9.android.comments.data.repository

import com.under9.android.comments.AbstractCommentSystemConfig
import com.under9.android.comments.AppOptions
import com.under9.android.comments.api.CommentCdnApiService

import com.under9.android.comments.controller.DataController
import com.under9.android.comments.data.processor.AppInfoProcessor
import com.under9.android.comments.data.query.info.AppInfoResult
import com.under9.android.lib.network.Transformations
import io.reactivex.Flowable

class AppInfoRepository(dataController: DataController,
                        private val api: CommentCdnApiService): BaseRepository(dataController), AppInfoRepositoryInterface {

    companion object {
        const val TAG = "AppInfoRepo"
    }

    override fun mayFetchInfo(force: Boolean): Flowable<AppInfoResult> {
        val lastInfoUpdateTime = getLastInfoUpdateTime()
        val now = System.currentTimeMillis()

        if (! (force || now - lastInfoUpdateTime > AbstractCommentSystemConfig.INFO_UPDATE_INTERVAL)) {
            return Flowable.just(AppInfoResult.getDefaultOrCached(dataController))
        }

        return api.getAppInfo(dataController.appId)
                .compose(Transformations.flowableResultToResponseWithWithHttpErrorHandling())
                .flatMap{
                    val apiInfo = it.body()

                    if (apiInfo == null) {
                        Flowable.just(AppInfoResult.getDefaultOrCached(dataController))
                    } else {
                        val appInfoProcess = AppInfoProcessor(dataController)
                        appInfoProcess.process(apiInfo)
                    }
                }
    }

    override fun allowAnonymousVote(): Boolean {
        return dataController.getIntData(AppOptions.FIELD_ALLOW_ANONYMOUS_VOTE, 0) == 1
    }

    override fun allowAnonymousComment(): Boolean {
        return dataController.getIntData(AppOptions.FIELD_ALLOW_ANONYMOUS_COMMENT, AbstractCommentSystemConfig.DEFAULT_ALLOW_ANONYMOUS_COMMENT) == 1
    }

    override fun mobileUsernameClickable(): Boolean {
        return dataController.getIntData(AppOptions.FIELD_USER_NAME_CLICKABLE, 0) == 1
    }

    override fun isEnableOpLabelFeature(): Boolean {
        return dataController.getIntData(AppOptions.FIELD_ENABLE_OP_LABEL_FEATURE, AbstractCommentSystemConfig.DEFAULT_ENABLE_OP_LABEL_FEATURE) == 1
    }

    override fun isEnableOpHideCommentFeature(): Boolean {
        return dataController.getIntData(AppOptions.FIELD_ENABLE_OP_HIDE_COMMENT_FEATURE, AbstractCommentSystemConfig.DEFAULT_ENABLE_OP_HIDE_COMMENT_FEATURE) == 1
    }

    override fun allowImageUploadFeature(): Boolean {
        return dataController.getIntData(AppOptions.FIELD_ALLOW_IMAGE_UPLOAD_FEATURE, AbstractCommentSystemConfig.DEFAULT_ALLOW_IMAGE_UPLOAD_FEATURE) == 1
    }

    override fun getAnonymousAvatarUrl(): String {
        return dataController.getStringData(AppOptions.FIELD_ANONYMOUS_AVATAR_URL)
    }

    override fun getMaxCommentLevel(): Int {
        return dataController.getIntData(AppOptions.FIELD_MAX_DISPLAY_LEVEL, AbstractCommentSystemConfig.DEFAULT_MAX_DISPLAY_LEVEL)
    }

    override fun getLastInfoUpdateTime(): Long {
        return dataController.getLongData(AppOptions.FIELD_LAST_INFO_UPDATE_TIME)
    }
}