package com.under9.android.comments.data.repository

import com.under9.android.comments.api.CommentApiService
import com.under9.android.comments.controller.DataController
import com.under9.android.comments.data.query.comment.CommentListQueryParam
import com.under9.android.comments.data.query.comment.CommentListQueryResult
import com.under9.android.comments.helper.CommentHelper
import com.under9.android.comments.helper.CommentWriterV2
import com.under9.android.lib.network.Transformations
import io.reactivex.Flowable
import io.reactivex.functions.Function
import org.reactivestreams.Publisher
import timber.log.Timber

class CommentListRepository(dataController: DataController,
                            private val api: CommentApiService): BaseRepository(dataController), CommentListRepositoryInterface {

    override fun getCommentList(commentListQueryParam: CommentListQueryParam): Flowable<CommentListQueryResult> {
        commentListQueryParam.run {
            return api.getCommentList(dataController.appId,
                    url,
                    CommentHelper.getOrderString(order),
                    count,
                    ref, refCommentId, direction, commentId)
                    .compose(Transformations.flowableResultToResponseWithWithHttpErrorHandling())
                    .flatMap {
                        val apiCommentList = it.body()

                        apiCommentList?.payload?.comments?.forEach { comment ->
                            Timber.d("api, subLevelLoadMore=$comment")
                        }


                        if (apiCommentList == null) {
                            Flowable.just(CommentListQueryResult(false, ""))
                        } else {
                            val processor = CommentListProcessor(dataController, commentListQueryParam, CommentWriterV2.instance)
                            processor.process(apiCommentList)
                        }
                    }
                    .onErrorResumeNext(Function<Throwable, Publisher<CommentListQueryResult>> {
                        Timber.e(it, "onErrorResumeNext")
                        Flowable.just(CommentListQueryResult(true, dataController.getLastOrderKey(commentListQueryParam.url, commentListQueryParam.order)))
                    })
        }
    }

}