package com.under9.android.comments.data.repository

import com.under9.android.comments.data.query.user.UserStatusQueryResult
import io.reactivex.Flowable

interface UserRepositoryInterface {
    /**
     * Fetch user's status
     *
     * 1. Comments he voted
     * 2. Post url he has commented on
     * 3. Post url he has reported
     *
     * @return Flowable<UserStatusQueryResult>
     */
    fun mayFetchUserStatus(): Flowable<UserStatusQueryResult>

    /**
     * Get the cached like status from like mapping
     * @see [mayFetchUserStatus]
     */
    fun getCommentLikeStatus(commentId: String): Int

    /**
     * @see [mayFetchUserStatus]
     */
    fun isPostReported(url: String): Boolean

    /**
     * @see [mayFetchUserStatus]
     */
    fun isPostCommented(url: String): Boolean

    /**
     * Updating the cached mapping when user has done voting
     */
    fun updateCommentLikeStatusOnCommentDbObject(likeMapping: Map<String, Int>?): Boolean

    /**
     *
     */
    fun addCommentedPost(postUrl: String)

    /**
     *
     */
    fun addReportedPost(postUrl: String)

    /**
     *
     */
    fun addLikeOrDislikedComment(commentId: String, likeStatus: Int)

    /**
     * Get the last API call time
     */
    fun getLastUserStatusQueryTime(): Long

    /**
     * Used in
     * @see com.under9.android.comments.model.ModelFactory
     */
    fun shouldOverrideCommentLikeStatus(): Boolean

    /**
     * Clear user data when logout
     */
    fun clear()
}