package com.under9.android.comments.model.api

import com.google.gson.JsonDeserializationContext
import com.google.gson.JsonElement
import com.google.gson.JsonParseException
import com.google.gson.reflect.TypeToken
import com.under9.android.comments.controller.MetricsController
import com.under9.android.comments.GsonFactory
import com.under9.android.lib.util.ApiPrimitiveTypeCheckDeserializer
import java.lang.reflect.Type

/**
 * Corresponding to the API user.json
 */
class ApiUserStatus: ApiResponse() {
    @JvmField var payload: Payload? = null

    class Payload {
        @JvmField var user: ApiUser? = null
        @JvmField var quota: ApiQuota? = null
        @JvmField var likeMapping: Map<String, Int>? = null
        @JvmField var commentedPostUrls: Map<String, Int>? = null
        @JvmField var reportedPostUrls: Map<String, Int>? = null

        override fun toString(): String {
            return "user={$user}, \n" +
                    "likeMapping={$likeMapping}, \n" +
                    "commentedPostUrls={$commentedPostUrls}, \n" +
                    "reportedPostUrls={$reportedPostUrls}"
        }
    }

    override fun toString(): String {
        return "payload={$payload}"
    }

    class ApiUserStatusPayloadDeserializer: ApiPrimitiveTypeCheckDeserializer<ApiUserStatus.Payload>() {
        private val mapType = object: TypeToken<HashMap<String, Int>>() {}.type

        override fun deserialize(json: JsonElement?, typeOfT: Type?, context: JsonDeserializationContext?): ApiUserStatus.Payload? {
            json?.run {
                if (!isJsonObject) {
                    MetricsController.logApiUnexpectedDataTypeInvalidJSONObject(toString())
                    return null
                }

                try {
                    val obj = asJsonObject
                    val gson = GsonFactory.getGson(GsonFactory.TYPE_STANDARD)
                    val apiUserStatusPayload = Payload()

                    val userElement = getObjectElementIfExist(obj, "user")
                    userElement?.let {
                        apiUserStatusPayload.user = gson.fromJson(it, ApiUser::class.java)
                    }

                    apiUserStatusPayload.run {
                        val likeMappingElement = getObjectElementIfExist(obj, "likeMapping")
                        likeMappingElement?.let {
                            likeMapping = gson.fromJson(it, mapType)
                        }

                        val commentPostUrlsElement = getObjectElementIfExist(obj, "commentedPostUrls")
                        commentPostUrlsElement?.let {
                            commentedPostUrls = gson.fromJson(it, mapType)
                        }

                        val reportedPostUrlsElement = getObjectElementIfExist(obj, "reportedPostUrls")
                        reportedPostUrlsElement?.let {
                            reportedPostUrls = gson.fromJson(it, mapType)
                        }
                    }

                    return apiUserStatusPayload

                } catch (e: JsonParseException) {

                }
            }

            return null
        }
    }
}