package com.under9.android.comments.model.ui

import androidx.collection.ArraySet
import com.under9.android.comments.controller.DataController
import com.under9.android.comments.data.repository.AppInfoRepositoryInterface
import com.under9.android.comments.model.wrapper.CommentWrapperV2
import com.under9.android.comments.ui.fragment.singlecomment.ExpandableCommentItemBinderProvider
import com.under9.android.comments.ui.renderer.CommentItemActionDelegate
import com.under9.android.comments.ui.renderer.ExpandableCommentItem
import com.under9.android.lib.network.UrlMapperInterface
import com.xwray.groupie.Section
import timber.log.Timber

internal class ExpandableCommentGroup(
        private val commentWrapper: CommentWrapperV2,
        private val dataController: DataController,
        private val appInfoRepository: AppInfoRepositoryInterface,
        private val urlMapperInterface: UrlMapperInterface,
        private val depth: Int = 0,
        private val commentItemThemeAttr: CommentItemThemeAttr,
        private val commentItemActionDelegate: CommentItemActionDelegate,
        private val expandableCommentItemBinderProvider: ExpandableCommentItemBinderProvider
): Section(
        ExpandableCommentItem(
                commentWrapper,
                depth,
                dataController,
                appInfoRepository,
                urlMapperInterface,
                commentItemThemeAttr,
                commentItemActionDelegate,
                expandableCommentItemBinderProvider
        )
) {

    val commentId: String = commentWrapper.getCommentId()
    private val childIds: ArraySet<String> by lazy(LazyThreadSafetyMode.NONE) { ArraySet<String>() }
    private val pendingCommentIds: ArraySet<Long> by lazy(LazyThreadSafetyMode.NONE) { ArraySet<Long>() }

    init {
        for (child in commentWrapper.getChildren()) {
            add(ExpandableCommentGroup(
                    child, dataController, appInfoRepository, urlMapperInterface,
                    depth.plus(1),
                    commentItemThemeAttr, commentItemActionDelegate,
                    expandableCommentItemBinderProvider))
            childIds.add(child.getCommentId())
        }
    }

    fun refreshChildren(commentWrapper: CommentWrapperV2) {
        Timber.d("originalChildren=${this.commentWrapper.getChildren().size}")
        for (child in commentWrapper.getChildren()) {
            if (!this.childIds.contains(child.getCommentId())) {
                add(ExpandableCommentGroup(
                        CommentWrapperV2(child.getComment()!!),
                        dataController,
                        appInfoRepository,
                        urlMapperInterface,
                        depth.plus(1),
                        commentItemThemeAttr, commentItemActionDelegate,
                        expandableCommentItemBinderProvider
                ))
                childIds.add(child.getCommentId())
            }
        }
        Timber.d("updatedChildren=${this.commentWrapper.getChildren().size}")
    }

    /**
     * A a pending comment to list
     */
    fun addPendingComment(commentWrapper: CommentWrapperV2, depth: Int) {
        this.commentWrapper.getChildren().add(commentWrapper)
        val g = ExpandableCommentGroup(
                commentWrapper,
                dataController,
                appInfoRepository,
                urlMapperInterface,
                depth,
                commentItemThemeAttr, commentItemActionDelegate,
                expandableCommentItemBinderProvider
        )
        add(g)
        Timber.d("commentWrapperPending=${commentWrapper.getId()}")
        pendingCommentIds.add(commentWrapper.getId())

        g.notifyChanged()
    }

    /**
     * We would like to refresh pending comment in UI
     * and must use the db ID to do so
     */
    fun updateChildByDbId(commentDbId: Long) {
        for (i in groupCount - 1 downTo 0) {
            val g = this.getGroup(i)
            if (g is ExpandableCommentGroup) {
                val wrapper = g.commentWrapper
                if (wrapper.getId() == commentDbId) {
                    wrapper.refreshPendingItem()
                    childIds.add(wrapper.getCommentId())
                    g.notifyChanged()
                    Timber.d("commentDbId=$commentDbId, current=${wrapper.getId()}, text=${wrapper.getComment()?.commentText}")
                    return
                }
            }
        }
    }

    /**
     * Remove the comment
     */
    fun removeChild(commentId: String) {
        for (i in 0 until groupCount) {
            val g = this.getGroup(i)
            if (g is ExpandableCommentGroup) {
                if (g.commentWrapper.getCommentId() == commentId) {
                    remove(g)
                    g.notifyChanged()
                    return
                }
            }
        }
    }
}