package com.under9.android.comments.model.wrapper

import com.under9.android.lib.widget.media.wrapper.IMediaWrapperV2

object CommentTransformer {

    @JvmStatic
    fun transform(commentWrapper: CommentWrapperInterface): IMediaWrapperV2? {

        if(commentWrapper.getEmbedMediaMeta() == null) return null

        val embedImage = commentWrapper.getEmbedMediaMeta()?.embedImage?.let {
                it
        } ?: run {
            commentWrapper.getFirstMediaEmbedImage()
        } ?: run{
            return null
        }

        with(embedImage) {
            if (image == null || type == null) return null

            when (type) {
                com.under9.android.comments.model.constant.CommentConstant.MEDIA_TYPE_STATIC -> {
                    return CommentTransformWrapper(
                            mediaHeight = image!!.height,
                            mediaWidth = image!!.width,
                            mediaId = commentWrapper.getCommentId(),
                            mediaTitle = commentWrapper.getTime().toString(),
                            embedImage = this,
                            commentType = type,
                            mediaImageUrl = embedImage.image!!.url,
                            mediaVideoUrl = null)
                }
                com.under9.android.comments.model.constant.CommentConstant.MEDIA_TYPE_ANIMATED -> {
                    return CommentTransformWrapper(
                            mediaHeight = image!!.height,
                            mediaWidth = image!!.width,
                            mediaId = commentWrapper.getCommentId(),
                            mediaTitle = commentWrapper.getTime().toString(),
                            embedImage = this,
                            commentType = type,
                            mediaImageUrl = null,
                            mediaVideoUrl = embedImage.mp4Url)
                }
                // case when the Comment Constant is progressing
                else -> return null
            }
        }

    }

}

class CommentTransformWrapper(val mediaVideoUrl : String?,
                              val mediaHeight: Int,
                              val mediaWidth: Int,
                              val mediaTitle: String,
                              val mediaImageUrl: String?,
                              val commentType: String,
                              val embedImage: EmbedImage,
                              val mediaId : String) : IMediaWrapperV2 {


    override fun getTitle(): String = mediaTitle

    override fun getId(): String = mediaId

    override fun getImageUrl() = mediaImageUrl

    override fun getImageWidth() = mediaWidth

    override fun getImageHeight() = mediaHeight

    override fun getVideoUrl() = mediaVideoUrl

    // animated is not a gif
    override fun isOtherVideo() = !embedImage.isAnimated && mediaVideoUrl != null

    override fun getMediaType() = commentType
}

