package com.under9.android.comments.ui.fragment

import android.app.Activity
import android.content.Context
import android.os.Bundle
import android.text.Html
import android.util.Log
import android.view.inputmethod.InputMethodManager
import android.widget.Toast
import com.under9.android.comments.controller.CommentSystemTaskQueueController
import com.under9.android.comments.controller.DataController
import com.under9.android.comments.helper.CommentActionHelper
import com.under9.android.comments.model.wrapper.CommentWrapper
import com.under9.android.comments.model.wrapper.CommentWrapperInterface
import com.under9.android.comments.ui.fragment.dialog.CommentBottomSheetBuilder
import com.under9.android.comments.ui.renderer.RendererItemActionListener
import com.under9.android.commentsystem.R
import com.under9.android.lib.bottomsheet.GagBottomSheetDialogFragment
import com.under9.android.lib.bottomsheet.MenuItemId
import com.under9.android.lib.bottomsheet.Position
import com.under9.android.lib.util.ClipBoardUtil
import com.under9.android.lib.widget.uiv3.adapter.UIVAdapter
import timber.log.Timber
import java.lang.ref.WeakReference

open class RendererItemActionListenerImpl(
        viewRef: CommentListingViewInterface
) : RendererItemActionListener {

    override var selectedCommentId: Long = -1

    private val viewRef: WeakReference<CommentListingViewInterface> = WeakReference(viewRef)
    private val controller: CommentSystemTaskQueueController = viewRef.getCommentSystemTaskQueueController()

    override fun shouldHightlightSelectedComment(): Boolean {
        val addModule = viewRef.get()?.getInlineAddModule()
        return addModule?.hasFocus() ?: false
    }

    override fun onAuthorNameClick(authorName: String) {
        // expose to app side
    }

    override fun onAuthorAvatarClick(authorName: String) {
        // expose to app side
    }

    override fun onMentionedUserClick(username: String, accountId: String) {
        // expose to app side
    }

    override fun onItemReply(position: Int, commentWrapper: CommentWrapperInterface?, prefill: String, suppData: Bundle?) {
        // expose to app side
    }

    /* TODO */
    override fun onLoadMorePrevItems(position: Int) {
        /*
        Log.d(TAG, "onLoadMorePrevItems=" + position);
        dispatchOnLoadMorePrevItemsEvent(position);
        */
    }

    override fun onItemMore(position: Int, commentWrapper: CommentWrapperInterface?, authorName: String) {


        viewRef.get()?.run {
            if (!isVisible()) return
//            val focusView = viewRef.get()?.getActivity()?.currentFocus
//            focusView?.let { v ->
//                val imm = viewRef.get()?.getActivity()?.getSystemService(Context.INPUT_METHOD_SERVICE) as? InputMethodManager
//                imm?.hideSoftInputFromWindow(v.windowToken, 0)
//            }

            commentWrapper?.let { commentWrapperInterface ->
                val comment = commentWrapperInterface.getComment()
                comment?.let {
                    val user = it.user
                    user?.let {
                        val isCommentCreator = CommentWrapper.isCommentCreator(comment, DataController.getInstance().loggedInUserId)
                        // Create with bottom sheet
                        val refComment = WeakReference(commentWrapper)
                        getActivity()?.let { activity ->
                            if (activity.isFinishing) return
                            try {
                                val moreActionListener = getMoreActionCallback()
                                val fragment = GagBottomSheetDialogFragment.newInstance(CommentBottomSheetBuilder.buildActionTextSheet(isOP(), isCommentCreator, activity)).also {
                                    hideKeyboard(activity)
                                }
                                fragment.setBottomSheetClickListener { pos: Position, id: MenuItemId ->
                                    refComment.get()?.let { wrapper ->
                                        when (id) {
                                            R.id.action_report_comment -> moreActionListener?.onItemReport(wrapper)
                                            R.id.action_delete_comment -> moreActionListener?.onItemDelete(wrapper)
                                            R.id.action_copy_comment_link -> moreActionListener?.onItemShare(wrapper)
                                            R.id.action_copy_comment -> moreActionListener?.onItemCopy(wrapper)
                                            else -> { }
                                        }
                                    }
                                }
                                fragment.show(getChildFragmentManager()!!, "more_action")
                            } catch (e: IllegalStateException) {
                                // Suppress IllegalStateException
                                // https://fabric.io/9gag2/android/apps/com.ninechat.android.chat/issues/579d2531ffcdc04250c8090b
                                Log.e("OnItemAction", e.message, e)
                            }
                        }
                    }
                }

            } ?: return
        }
    }

    private fun hideKeyboard(activity: Activity){
        val focusView =  activity.currentFocus
        focusView?.let { v ->
            val imm = activity.getSystemService(Context.INPUT_METHOD_SERVICE) as? InputMethodManager
            imm?.hideSoftInputFromWindow(v.windowToken, 0)
        }
    }

    override fun onItemLike(position: Int, commentId: String): Boolean {
        viewRef.get()?.run {
            CommentActionHelper.like(controller, commentId, getUrl())
        }
        return false
    }

    override fun onItemDislike(position: Int, commentId: String): Boolean {
        viewRef.get()?.run {
            if (!isVisible()) return false
            CommentActionHelper.dislike(controller, commentId, getUrl())
        }
        return false
    }

    override fun onItemUnlike(position: Int, commentId: String): Boolean {
        viewRef.get()?.run {
            if (!isVisible()) return false
            CommentActionHelper.unlike(controller, commentId, getUrl())
        }
        return false
    }

    override fun onLongClickMedia(position: Int, commentWrapper: CommentWrapperInterface?): Boolean {
        return false
    }

    override fun onItemOpenOverlay(commentWrapper: CommentWrapperInterface?, uivAdapter: UIVAdapter) {
        // expose to app side
    }

    override fun onItemReport(position: Int, commentWrapper: CommentWrapperInterface?) {
        viewRef.get()?.run {
            commentWrapper?.let {
                try {
                    val activity = getActivity() ?: return
                    val childFragmentManager = getChildFragmentManager() ?: return

                    CommentActionHelper.report(activity, childFragmentManager, getCommentSystemTaskQueueController(), getUrl(), commentWrapper.getCommentId(), getUrl())
                } catch (e: Exception) {
                    Timber.w(e)
                }
            }
        }
    }

    override fun onItemReportAccepted(commentWrapper: CommentWrapperInterface?) {
        // expose to app side
    }

    override fun onItemHide(position: Int, commentWrapper: CommentWrapperInterface?) {
        viewRef.get()?.run {
            commentWrapper?.let {
                try {
                    val activity = getActivity() ?: return
                    val childFragmentManager = getChildFragmentManager() ?: return

                    CommentActionHelper.hide(activity, childFragmentManager, getCommentSystemTaskQueueController(), getUrl(), commentWrapper.getCommentId(), getUrl())
                } catch (e: Exception) {
                    Timber.w(e)
                }
            }
        }
    }

    override fun onItemDelete(position: Int, commentWrapper: CommentWrapperInterface?) {
        viewRef.get()?.run {
            commentWrapper?.let {
                try {
                    val activity = getActivity()
                    val childFragmentManager = getChildFragmentManager() ?: return

                    if (!CommentActionHelper.isDeletable(commentWrapper.getCommentId())) {
                        if (activity != null && !activity.isFinishing) {
                            Toast.makeText(activity, R.string.fail_delete_with_reply, Toast.LENGTH_LONG).show()
                        }
                        return
                    }

                    CommentActionHelper.delete(activity, childFragmentManager, getCommentSystemTaskQueueController(), getUrl(), commentWrapper.getCommentId(), getUrl())
                } catch (e: Exception) {
                    Timber.w(e)
                }
            }
        }
    }

    override fun onRefresh(position: Int, commentWrapper: CommentWrapperInterface?) {

    }

    override fun onItemCopy(commentWrapper: CommentWrapperInterface?) {
        viewRef.get()?.getActivity().let { activity ->
            commentWrapper?.let {
                val text = commentWrapper.getComment()?.commentText ?: ""
                val htmlText = Html.fromHtml(text.replace("\n", "<br/>")).toString()
                ClipBoardUtil.simpleClip(activity, htmlText, "9GAG")
                Toast.makeText(activity, R.string.comment_copied, Toast.LENGTH_LONG).show()
            }
        }
    }

    override fun onItemShare(commentWrapper: CommentWrapperInterface?) {
        viewRef.get()?.getActivity().let { activity ->
            commentWrapper?.let {
                ClipBoardUtil.simpleClip(activity, CommentActionHelper.getShareUrl(commentWrapper.getCommentId()), "9GAG")
                Toast.makeText(activity, R.string.comment_shared, Toast.LENGTH_SHORT).show()
            }
        }
    }

    override fun onItemProBadge(commentWrapper: CommentWrapperInterface?) {
        // expose to app side
    }

    override fun onItemUrl(commentWrapper: CommentWrapperInterface?, url: String) {
        // expose to app side
    }
}
