package com.under9.android.comments.ui.fragment.singlecomment

import android.annotation.SuppressLint
import androidx.lifecycle.Lifecycle
import androidx.lifecycle.LifecycleObserver
import androidx.lifecycle.OnLifecycleEvent
import android.content.BroadcastReceiver
import android.content.Context
import android.content.Intent
import android.content.IntentFilter
import com.hwangjr.rxbus.annotation.Subscribe
import com.jakewharton.rxrelay2.PublishRelay
import com.under9.android.comments.CommentSystem
import com.under9.android.comments.controller.DataController
import com.under9.android.comments.event.AddCommentFailedEvent
import com.under9.android.comments.event.AddCommentQuotaExceededEvent
import com.under9.android.comments.event.RequestAddCommentEvent
import com.under9.android.comments.model.PendingCommentListItem
import com.under9.android.comments.model.wrapper.CommentWrapperV2
import com.under9.android.comments.service.TaskQueueService
import com.under9.android.comments.task.AddCommentTask
import com.under9.android.comments.task.Task
import com.under9.android.lib.core.mvvm.RxViewModel
import timber.log.Timber

class ComposerModuleViewModel(context: Context,
                              dataController: DataController,
                              val url: String) : RxViewModel(), LifecycleObserver {

    val requestAddCommentRelay = PublishRelay.create<CommentWrapperV2>()
    val commentAddedRelay = PublishRelay.create<Long>()

    @SuppressLint("StaticFieldLeak")
    private val context: Context = context.applicationContext
    private val intentFilter: IntentFilter = IntentFilter(CommentSystem.getInstance().API_CALLBACK)
    private val receiver: BroadcastReceiver = object: BroadcastReceiver() {
        override fun onReceive(context: Context?, intent: Intent?) {
            intent?.let {
                when (intent.getIntExtra(TaskQueueService.PARAM_COMMAND, -1)) {
                    Task.COMMAND_ADD_COMMENT -> {
                        val commentDbId: Long = intent.getLongExtra(AddCommentTask.KEY_PENDING_COMMENT_DB_ID, -1)
                        val success: Boolean = intent.getBooleanExtra("success", false)
                        if (success) {
                            commentAddedRelay.accept(commentDbId)
                        }
                    }
                }
            }
        }
    }

    @OnLifecycleEvent(Lifecycle.Event.ON_START)
    fun subscribe() {
        CommentSystem.registerEventBus(url, this)
        context.registerReceiver(receiver, intentFilter)
    }

    @OnLifecycleEvent(Lifecycle.Event.ON_STOP)
    fun unsubscribe() {
        CommentSystem.unregisterEventBus(url, this)
        context.unregisterReceiver(receiver)
    }

    @OnLifecycleEvent(Lifecycle.Event.ON_DESTROY)
    override fun onCleared() {
        super.onCleared()
    }

    @Subscribe
    fun onRequestAddComment(event: RequestAddCommentEvent) {
        Timber.d("event=$event")
        event.pendingCommentListItem?.let {
            requestAddCommentRelay.accept(CommentWrapperV2(it))
        }
    }

    @Subscribe
    fun onAddCommentFailed(event: AddCommentFailedEvent) {
        Timber.d("onAddCommentFailed, event=$event")
    }

    @Subscribe
    fun onAddCommentQuotaExceeded(event: AddCommentQuotaExceededEvent) {
        Timber.d("onAddCommentQuotaExceeded, event=$event")
    }
}