package com.under9.android.comments.ui.fragment.singlecomment

import com.under9.android.comments.controller.DataController
import com.under9.android.comments.data.repository.AppInfoRepositoryInterface
import com.under9.android.comments.model.constant.CommentConstant
import com.under9.android.comments.ui.renderer.*
import com.under9.android.lib.network.DomainMapperModule
import com.under9.android.lib.network.UrlMapperInterface
import com.xwray.groupie.ViewHolder

class ExpandableCommentItemBinderProvider {
    private lateinit var defaultCommentItemItemBinder: DefaultCommentItemItemBinder
    private lateinit var mediaCommentItemItemBinder: MediaCommentItemItemBinder
    private lateinit var userMediaCommentItemItemBinder: UserMediaCommentItemItemBinder

    private lateinit var commentItemClickListener: CommentItemClickListener

    fun obtainBinder(dataController: DataController,
                     appInfoRepository: AppInfoRepositoryInterface,
                     urlMapperInterface: UrlMapperInterface,
                     commentItemActionDelegate: CommentItemActionDelegate,
                     type: Int = -1
    ): ExpandableCommentItemBinder<ViewHolder> {
        if (!::commentItemClickListener.isInitialized) {
            commentItemClickListener = CommentItemClickListener(commentItemActionDelegate, appInfoRepository, true)
        }

        val itemBinder: ExpandableCommentItemBinder<ViewHolder>
        when (type) {
            CommentConstant.TYPE_USER_MEDIA -> {
                if (!::userMediaCommentItemItemBinder.isInitialized) {
                    userMediaCommentItemItemBinder =
                            UserMediaCommentItemItemBinder(dataController, appInfoRepository,
                                    urlMapperInterface, commentItemActionDelegate,
                                    commentItemClickListener)
                }
                itemBinder = userMediaCommentItemItemBinder
            }

            CommentConstant.TYPE_MEDIA -> {
                if (!::mediaCommentItemItemBinder.isInitialized) {
                    mediaCommentItemItemBinder =
                            MediaCommentItemItemBinder(dataController, appInfoRepository,
                                    urlMapperInterface, commentItemActionDelegate,
                                    commentItemClickListener)
                }
                itemBinder = userMediaCommentItemItemBinder
            }

            else -> {
                if (!::defaultCommentItemItemBinder.isInitialized) {
                    defaultCommentItemItemBinder =
                            DefaultCommentItemItemBinder(dataController,
                                    appInfoRepository, commentItemActionDelegate, commentItemClickListener)
                }
                itemBinder = defaultCommentItemItemBinder
            }
        }

        return itemBinder
    }
}