package com.under9.android.comments.ui.fragment.singlecomment

import android.app.Activity
import android.content.Intent
import android.os.Bundle
import android.text.TextUtils
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.annotation.LayoutRes
import androidx.fragment.app.Fragment
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.recyclerview.widget.SimpleItemAnimator
import com.ninegag.android.blitz2.BlitzActionListener
import com.ninegag.android.blitz2.BlitzState
import com.ninegag.android.blitz2.BlitzView
import com.ninegag.android.blitz2.BlitzViewConfig
import com.ninegag.android.blitz2.ui.delegate.LinearPaginationDetector
import com.under9.android.comments.CommentSystem
import com.under9.android.comments.controller.CommentSystemTaskQueueController
import com.under9.android.comments.controller.DataController
import com.under9.android.comments.controller.RepositoryManager
import com.under9.android.comments.data.query.comment.CommentListQueryParam
import com.under9.android.comments.data.repository.AppInfoRepositoryInterface
import com.under9.android.comments.data.repository.CommentListRepositoryInterface
import com.under9.android.comments.data.repository.UserRepositoryInterface
import com.under9.android.comments.model.Comment
import com.under9.android.comments.model.ui.CommentItemThemeAttr
import com.under9.android.comments.model.ui.ExpandableCommentGroup
import com.under9.android.comments.model.ui.LoadingIndicatorItem
import com.under9.android.comments.model.wrapper.CommentWrapper
import com.under9.android.comments.model.wrapper.CommentWrapperV2
import com.under9.android.comments.ui.fragment.BaseCommentSystemInlineAddModule
import com.under9.android.comments.ui.fragment.CommentListingViewInterface
import com.under9.android.comments.ui.fragment.CommentSystemInlineAddModuleV2
import com.under9.android.comments.ui.fragment.DefaultMoreActionCallback
import com.under9.android.comments.ui.fragment.dialog.MoreActionDropdownDialogFragment
import com.under9.android.comments.ui.renderer.CommentItemActionDelegate
import com.under9.android.comments.ui.renderer.RendererConfig
import com.under9.android.comments.ui.renderer.RendererItemActionListener
import com.under9.android.commentsystem.R
import com.under9.android.lib.util.IListItemFinder
import com.under9.android.lib.util.RecyclerViewListItemFinder
import com.under9.android.lib.widget.inlinecomposer.ComposerView
import com.under9.android.lib.widget.uiv2.mp4.listener.AutoPlayListener
import com.under9.android.lib.widget.uiv3.ui.PlaybackControlInterface
import com.under9.android.lib.widget.wizard.SnackbarView
import com.xwray.groupie.GroupAdapter
import com.xwray.groupie.ViewHolder
import timber.log.Timber
import java.lang.ref.WeakReference

abstract class SingleSublevelCommentFragment : Fragment(), CommentListingViewInterface {

    companion object {
        const val KEY_QUERY_PARAM = "query_param"
        const val KEY_URL = "url"
        const val KEY_POST_ID = "post_id"
        const val KEY_IS_OP = "is_op"

        // More menu
        const val KEY_SHOW_UPVOTE_OPTION = "show_upvote"
        const val KEY_SHOW_DOWNVOTE_OPTION = "show_downvote"
        const val KEY_SHOW_REPLY = "show_reply"
        const val KEY_SHOW_SHARE = "show_share"
        const val KEY_SHOW_GO_PROFILE = "show_go_profile"
        const val KEY_SHOW_HIDE_COMMENT = "show_hide_comment"

        const val KEY_PREFILL = "prefill"
        const val KEY_PARENT_COMMENT_ID = "parent_comment_id"
    }

    private var group: ExpandableCommentGroup? = null
    private val adapter = GroupAdapter<ViewHolder>()
    private val loadingIndicatorItem = LoadingIndicatorItem()
    private val commentItemThemeAttr = CommentItemThemeAttr()
    private val expandableCommentItemBinderProvider = ExpandableCommentItemBinderProvider()
    private val commentItemActionDelegate =
            CommentItemActionDelegate(DataController.getInstance(), commentItemThemeAttr, getRendererConfig())

    private var rendererConfig: RendererConfig? = null
    private var isOP: Boolean = false
    private var showUpvote: Boolean = false
    private var showDownvote: Boolean = false
    private var showReply: Boolean = false

    private lateinit var url: String
    private lateinit var postId: String
    private var commentSystemActionListener: CommentSystem.CommentSystemActionListener? = null
    private lateinit var onItemActionListener: com.under9.android.comments.ui.renderer.RendererItemActionListener
    private lateinit var moreActionCallback: DefaultMoreActionCallback
    private lateinit var blitzView: BlitzView
    private lateinit var viewModel: SingleSublevelCommentListViewModel
    private lateinit var composerModuleViewModel: ComposerModuleViewModel
    private lateinit var commentAddModule: BaseCommentSystemInlineAddModule
    private lateinit var commentSystemTaskQueueController: CommentSystemTaskQueueController
    private lateinit var autoPlayListener: AutoPlayListener<CommentWrapperV2>
    private lateinit var listItemFinder: IListItemFinder<CommentWrapperV2>
    protected var isEnabledAutoPlayGIF = false

    protected val commentSystemController by lazy(LazyThreadSafetyMode.NONE) {
        viewModel.commentSystemController
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)

        Timber.d("queryParam=${arguments?.get(KEY_QUERY_PARAM)}, prefill=${arguments?.get(KEY_PREFILL)}")

        arguments?.let {
            val queryParam = it.get(KEY_QUERY_PARAM) as CommentListQueryParam
            isEnabledAutoPlayGIF = it.getBoolean(CommentSystem.KEY_AUTOPLAY_GIF)
            url = queryParam.url
            postId = it.get(KEY_POST_ID) as String
            // TODO: 2/1/2019 Can think about Dagger later by using
            viewModel = createViewModel(
                    url,
                    CommentSystem.getInstance(),
                    RepositoryManager.userRepository,
                    RepositoryManager.appInfoRepository,
                    RepositoryManager.commentListRepository,
                    createCommentSystemActionListener(),
                    DataController.getInstance(),
                    queryParam
            )
            composerModuleViewModel = ComposerModuleViewModel(
                    context!!,
                    DataController.getInstance(),
                    url
            )
            lifecycle.addObserver(viewModel)
            lifecycle.addObserver(composerModuleViewModel)

            commentItemThemeAttr.timeFormatMode = it.getInt(CommentSystem.KEY_TIME_FORMAT_MODE, CommentSystem.TIME_FORMAT_MODE_ORIGINAL)
        }
    }

    override fun onStart() {
        super.onStart()
        commentSystemController.onStart(url)
        commentSystemController.setCommentSystemActionListener(commentSystemActionListener)
        commentAddModule.onStart()
    }

    override fun onStop() {
        super.onStop()
        commentAddModule.onStop()
        commentSystemController.setCommentSystemActionListener(null)
        commentSystemController.onStop(url)
    }

    override fun onResume() {
        super.onResume()
        commentAddModule.onResume()
    }

    override fun onPause() {
        super.onPause()
        commentAddModule.onPause()
    }

    override fun onDestroy() {
        if (::viewModel.isInitialized) {
            lifecycle.removeObserver(viewModel)
        }

        if (::composerModuleViewModel.isInitialized) {
            lifecycle.removeObserver(composerModuleViewModel)
        }
        if (::autoPlayListener.isInitialized) {
            autoPlayListener.disposeAll()
        }
        commentAddModule.onDestroy()

        super.onDestroy()
    }


    @LayoutRes
    protected open fun getLayoutId(): Int {
        return R.layout.comment_system_view_single_sublevel
    }

    /**
     *
     */
    abstract fun loadCommentAddModuleConfig(activity: Activity)

    /**
     *
     */
    @LayoutRes
    abstract fun getCommentAddModuleLayoutId(): Int

    /**
     *
     */
    abstract fun createCommentSystemActionListener(): CommentSystem.CommentSystemActionListener?

    open fun createViewModel(url: String,
                        commentSystem: CommentSystem,
                        userRepository: UserRepositoryInterface,
                        appInfoRepository: AppInfoRepositoryInterface,
                        commentListRepository: CommentListRepositoryInterface,
                        commentSystemActionListener: CommentSystem.CommentSystemActionListener?,
                        dataController: DataController,
                        queryParam: CommentListQueryParam): SingleSublevelCommentListViewModel {
        return SingleSublevelCommentListViewModel(
                url,
                commentSystem,
                userRepository,
                appInfoRepository,
                commentListRepository,
                commentSystemActionListener,
                dataController,
                queryParam
        )
    }

    private fun bindCommentAddUi(view: View) {
        val composerView = view.findViewById<ComposerView>(R.id.comment_inline_composer)
        composerView.setLayoutResId(getCommentAddModuleLayoutId())
        composerView.visibility = View.VISIBLE
        loadCommentAddModuleConfig(view.context as Activity)
        commentAddModule.setView(composerView)
    }

    override fun onCreateView(inflater: LayoutInflater,
                              container: ViewGroup?, savedInstanceState: Bundle?): View? {
        return inflater.inflate(getLayoutId(), container, false)
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)

        arguments?.let {
            commentSystemActionListener = createCommentSystemActionListener()
            onItemActionListener = createRendererItemActionListener(activity!!)
            moreActionCallback = DefaultMoreActionCallback(this, onItemActionListener)
            commentAddModule = createCommentSystemInlineAddModule(activity!!, it)
            commentAddModule.onCreate(savedInstanceState)
        }

        blitzView = view.findViewById(R.id.blitz)
        val config = BlitzViewConfig.Builder.newBuilder()
                .setAdapter(this.adapter)
                .setLayoutManager(LinearLayoutManager(context))
                .setRefreshListener(viewModel::refreshComment)
                .setPaginationDetector(LinearPaginationDetector(object : BlitzActionListener {
                    override fun onTriggerLoadNext() {
                        viewModel.loadNext()
                    }

                    override fun onTriggerLoadPrevious() {

                    }
                }, 20, false))
                .disableDefaultViewState()
                .setCustomBlitzViewStateCallback(viewModel.getBlitzViewStateCallback())
                .build()
        blitzView.setConfig(config)
        (blitzView.recyclerView.itemAnimator as SimpleItemAnimator).supportsChangeAnimations = false

        commentItemActionDelegate.apply {
            setRendererItemActionListener(onItemActionListener)
        }
        bindCommentAddUi(view)

        arguments?.let {
            viewModel.loadInitial()
            viewModel.disposables.add(
                    viewModel.reportedCommentRelay.subscribe { wrapper ->
                        onItemActionListener.onItemReportAccepted(wrapper)
                    }
            )

            viewModel.disposables.add(
                    viewModel.queryResultRelay.subscribe { queryResult ->
                        Timber.d("queryResult@Subscribe=$queryResult")

                        if (queryResult.commentWrapper == null) return@subscribe

                        if (queryResult.isRefresh) {
                            this.adapter.remove(group!!)
                            group = null
                        }

                        if (!queryResult.isLoadNext) {
                            if (group == null) {
                                group = ExpandableCommentGroup(
                                        queryResult.commentWrapper,
                                        DataController.getInstance(),
                                        RepositoryManager.appInfoRepository,
                                        CommentSystem.getInstance().config.domainMapperModule,
                                        0,
                                        commentItemThemeAttr,
                                        commentItemActionDelegate,
                                        expandableCommentItemBinderProvider
                                )
                                this.adapter.add(group!!)
                            } else {
                                group?.run {
                                    refreshChildren(queryResult.commentWrapper)
                                    notifyChanged()
                                }
                            }
                        } else {
                            group?.run {
                                refreshChildren(queryResult.commentWrapper)
                                notifyChanged()
                            }
                        }
                    }
            )

            viewModel.disposables.add(
                    viewModel.blitzViewStateRelay.subscribe { state ->
                        Timber.d("blitzView.viewState=$state, thread=${Thread.currentThread()}")
                        blitzView.updateViewState(state)
                        if (viewModel.isCommentWrapperInited()) {
                            if (!::autoPlayListener.isInitialized) {
                                autoPlayListener = createAutoPlayListener()
                                blitzView.addOnScrollListener(autoPlayListener)
                            } else {
                                (listItemFinder as RecyclerViewListItemFinder).updateItemList(viewModel.commentWrappers.getChildren())
                            }
                        }
                        when (state) {
                            BlitzState.STATE_FINISH_LOADING_HAS_NEXT -> {
                                if (adapter.getAdapterPosition(loadingIndicatorItem) != -1) {
                                    adapter.remove(loadingIndicatorItem)
                                }

                                adapter.add(loadingIndicatorItem)
                                blitzView.setRefreshing(false)
                                Timber.d("blitzView.setRefreshing, STATE_FINISH_LOADING_HAS_NEXT")
                            }
                            BlitzState.STATE_FINISH_LOADING_NEXT_END_OF_LIST -> {
                                if (adapter.getAdapterPosition(loadingIndicatorItem) != -1) {
                                    adapter.remove(loadingIndicatorItem)
                                }
                                blitzView.setRefreshing(false)
                                Timber.d("blitzView.setRefreshing, STATE_FINISH_LOADING_NEXT_END_OF_LIST")
                            }
                            else -> { }
                        }
                    }
            )

            viewModel.disposables.add(
                    viewModel.removeCommentRelay.subscribe { commentId ->
                        this.group?.let { expandableCommentGroup ->
                            if (expandableCommentGroup.commentId == commentId) {
                                activity?.finish()
                            }

                            expandableCommentGroup.removeChild(commentId)
                        }
                    }
            )

            viewModel.disposables.add(
                    viewModel.selectCommentRelay.subscribe { event ->
                        onItemActionListener.selectedCommentId = event.commentId
                        blitzView.post {
                            if (event.pos == 0) {
                                this.group?.notifyChanged()
                            } else {
                                (blitzView.layoutManager as LinearLayoutManager)
                                        .scrollToPositionWithOffset(event.pos, 0)
                                this.group?.notifyChanged()
                            }
                        }
                    }
            )

            composerModuleViewModel.disposables.add(
                    composerModuleViewModel.requestAddCommentRelay.subscribe { wrapper ->
                        Timber.d("pendingWrapper=$wrapper, itemCount=${group?.itemCount}")
                        group?.run {
                            if (!viewModel.listHasNext()) {
                                addPendingComment(wrapper, 1)
                                blitzView.post {
                                    blitzView.scrollToPosition(itemCount - 1)
                                }
                            }
                        }
                    }
            )

            composerModuleViewModel.disposables.add(
                    composerModuleViewModel.commentAddedRelay.subscribe { result ->
                        Timber.d("commentAddedRelay=$result")
                        this.group?.run {
                            updateChildByDbId(result)
                        }

                        activity?.let {
                            SnackbarView<String>("", it.findViewById(android.R.id.content),
                                    getString(R.string.comment_replyAdded)).show()
                        }
                    }
            )
        }
    }

    override fun onSaveInstanceState(outState: Bundle) {
        super.onSaveInstanceState(outState)
        commentAddModule.onSaveInstanceState(outState)
    }

    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        super.onActivityResult(requestCode, resultCode, data)
        commentAddModule.onActivityResult(requestCode, resultCode, data)
    }

    fun getRendererConfig(): RendererConfig {
        if (rendererConfig == null) {
            rendererConfig = RendererConfig()
        }
        return rendererConfig!!
    }

    fun setRendererConfig(rendererConfig: RendererConfig) {
        this.rendererConfig = rendererConfig
    }

    protected abstract fun createRendererItemActionListener(activity: Activity): RendererItemActionListener
    protected open fun createCommentSystemInlineAddModule(activity: Activity, arguments: Bundle): BaseCommentSystemInlineAddModule {
        val module = CommentSystemInlineAddModuleV2(activity, this, commentSystemController)
        module.arguments = arguments
        return module
    }

    override fun showUpvoteOption(): Boolean {
        return this.showUpvote
    }

    override fun showDownvoteOption(): Boolean {
        return this.showDownvote
    }

    override fun showReplyOption(): Boolean {
        return this.showReply
    }

    override fun showShare(comment: Comment): Boolean {
        return !TextUtils.isEmpty(comment.permalink)
    }

    override fun showGoProfile(comment: Comment): Boolean {
        return !CommentWrapper.isAnonymous(comment)
    }

    override fun showHideComment(): Boolean {
        return isOP() && RepositoryManager.appInfoRepository.isEnableOpHideCommentFeature()
    }

    override fun isOP(): Boolean {
        return this.isOP
    }

    override fun getUrl(): String {
        return this.url
    }

    fun getPostId(): String? {
        return this.postId
    }

    override fun getMoreActionCallback(): MoreActionDropdownDialogFragment.MoreActionCallback? {
        return moreActionCallback
    }

    override fun getInlineAddModule(): BaseCommentSystemInlineAddModule? {
        return commentAddModule
    }

    override fun getCommentSystemTaskQueueController(): CommentSystemTaskQueueController {
        if (!::commentSystemTaskQueueController.isInitialized) {
            commentSystemTaskQueueController = viewModel.commentSystemTaskQueueController
        }
        return commentSystemTaskQueueController
    }

    override fun getRendererItemActionListener(): RendererItemActionListener? {
        return onItemActionListener
    }

    private fun createAutoPlayListener() : AutoPlayListener<CommentWrapperV2> {
        listItemFinder = RecyclerViewListItemFinder(blitzView.recyclerView,
                viewModel.commentWrappers.getChildren())

        return AutoPlayListener(AutoPlayListener.SCROLLING_TO_PLAY_MEDIA,
                blitzView.context,
                listItemFinder as RecyclerViewListItemFinder<CommentWrapperV2>,
                createViewAutoPlayPredicate(),
                AutoPlayListener.TYPE_UPPER_TRIGGER)
    }
    private fun createViewAutoPlayPredicate(): AutoPlayListener.ViewAutoPlayPredicate{
        val refViewModel = WeakReference(viewModel)
        return object : AutoPlayListener.ViewAutoPlayPredicate {
            override fun shouldCheckView(position: Int): Boolean {
                if (!isEnabledAutoPlayGIF) return false
                val vm = refViewModel.get() ?: return false
                if (!vm.isCommentWrapperInited()) return false

                val wrappers = vm.commentWrappers
                if (position < 0 || position >= wrappers.getChildren().size) return false
                val wrapper = wrappers.getChildren()[position]

                val embedImage = wrapper.getEmbedMediaMeta()?.embedImage?.let {
                    it
                } ?: run {
                    wrapper.getFirstMediaEmbedImage()
                } ?: run {
                    return false
                }
                return embedImage.isAnimated
            }

            override fun getUIV(view: View?): PlaybackControlInterface? {
                return if (view != null) {
                    val uiv = view.findViewById<View>(R.id.image)
                    if (uiv is PlaybackControlInterface) uiv else null
                } else null
            }

            override val prependAdapterOffset: Int
                get() = 1 // Exclude to check auto play in the first comment
        }
    }
}