package com.under9.android.comments.ui.helper

import android.content.Context
import android.util.DisplayMetrics
import android.view.View
import android.view.WindowManager
import androidx.recyclerview.widget.RecyclerView
import com.under9.android.comments.model.wrapper.CommentWrapper
import com.under9.android.lib.util.ListItemFindingStrategy
import io.reactivex.processors.PublishProcessor
import timber.log.Timber

abstract class BaseCommentListItemFindingStrategy : ListItemFindingStrategy<CommentWrapper> {

    private var prependAdapterCount: Int = 0
    private var upperBound: Int = 0
    private var lowerBound: Int = 0
    protected var key: String = ""

    constructor(context: Context, key: String) : this(context, key, 0)

    constructor(context: Context, key: String, prependAdapterCount: Int) {
        val metrics = DisplayMetrics()
        val wm = context.getSystemService(Context.WINDOW_SERVICE) as WindowManager
        wm.defaultDisplay.getMetrics(metrics)

        this.upperBound = metrics.heightPixels / 4
        this.lowerBound = metrics.heightPixels / 4 * 3
        this.prependAdapterCount = prependAdapterCount
        this.key = key
    }

    override fun shouldCheckView(v: View?, position: Int): Boolean {
        return true
    }

    override fun getPrependAdapterOffset(): Int {
        return prependAdapterCount
    }

    override fun onViewPassed(v: View?, visiblePos: Int, position: Int) {
        Timber.d("$v")
    }

    override fun onItemPositionOutOfBound(v: View?, visiblePos: Int, position: Int) {
        Timber.d("$v")
    }

    override fun getUpperLine(): Int {
        return upperBound
    }

    override fun getLowerLine(): Int {
        return lowerBound
    }
}