package com.under9.android.comments.ui.helper

import android.content.Context
import android.view.View
import android.widget.AbsListView
import com.under9.android.comments.helper.ProxyArrayList
import com.under9.android.comments.model.wrapper.CommentWrapper
import com.under9.android.comments.ui.AutoPlayCommentListViewListItemFinder
import com.under9.android.commentsystem.R
import com.under9.android.lib.widget.mp4.Mp4PlayerViewCompat
import com.under9.android.lib.widget.uiv2.mp4.listener.AutoPlayDelegate
import com.under9.android.lib.widget.uiv3.ui.PlaybackControlInterface
import io.reactivex.disposables.CompositeDisposable
import io.reactivex.disposables.Disposable

/**
 * Created by raymond on 22/11/2017.
 */
// TODO This implmentation is duplicate from AutoPlayListener, should use that class after migrate to recyclerview view
class CommentListItemAutoPlayStrategy(context: Context,
                                      key: String,
                                      prependAdapterCount: Int,
                                      private val autoPlayMode: Int,
                                      listItemFinder: AutoPlayCommentListViewListItemFinder<ProxyArrayList<CommentWrapper>, CommentWrapper>,
                                      private val isEnabledAutoPlayGif: Boolean)
    : BaseCommentListItemFindingStrategy(context, key, prependAdapterCount), AbsListView.OnScrollListener {

    private var lastFoundItemPos = -1
    val compositeDisposable = CompositeDisposable()
    private val autoPlayDelegate: AutoPlayDelegate<CommentWrapper> = AutoPlayDelegate(listItemFinder, autoPlayMode, this)

    init {
        autoPlayDelegate.autoPlayDisposable?.let {
            addDisposable(it)
        }
    }

    override fun shouldCheckView(v: View?, position: Int): Boolean {
        return isEnabledAutoPlayGif
    }

    override fun onViewPassed(v: View?, visiblePos: Int, position: Int) {
        super.onViewPassed(v, visiblePos, position)
        if (v == null) return
        val uiv = v.findViewById<View>(R.id.image)

        if (uiv is PlaybackControlInterface) {
            if (position == lastFoundItemPos) {
                lastFoundItemPos = -1
                uiv.stop()
            }
        }
    }

    override fun onItemFound(v: View?, visiblePos: Int, position: Int, commentWrapper: CommentWrapper) {
        if (v == null) return
        if (commentWrapper.getEmbedMediaMeta() == null) return

        val embedImage = commentWrapper.getEmbedMediaMeta()?.embedImage?.let {
            it
        } ?: run {
            commentWrapper.getFirstMediaEmbedImage()
        } ?: run {
            return
        }
        val uiv = v.findViewById<View>(R.id.image)
        if (uiv is PlaybackControlInterface && embedImage.isAnimated) {
            if (lastFoundItemPos != position || ((lastFoundItemPos == position) && !Mp4PlayerViewCompat.isPlaying(uiv))) {
                uiv.postDelayed({
                    uiv.play()
                }, 100)
                lastFoundItemPos = position
            }
        }
    }

    override fun onScroll(view: AbsListView?, firstVisibleItem: Int, visibleItemCount: Int, totalItemCount: Int) {
        autoPlayDelegate.onScroll(AutoPlayDelegate.SCROLL_WITHOUT_DELTA)
    }

    override fun onScrollStateChanged(view: AbsListView?, scrollState: Int) {
    }


    fun addDisposable(disposable: Disposable) = compositeDisposable.add(disposable)

    fun checkAutoPlay() {
        autoPlayDelegate.onScroll(AutoPlayDelegate.SCROLL_WITHOUT_DELTA)
    }
}
