package com.under9.android.comments.ui.helper

import android.content.Context
import android.view.View
import android.widget.ListView
import com.under9.android.comments.AppOptions
import com.under9.android.comments.controller.DataController
import com.under9.android.comments.model.wrapper.CommentWrapper
import com.under9.android.lib.util.StringUtils
import timber.log.Timber

class CommentListItemScrollPositionStrategy : BaseCommentListItemFindingStrategy {

    companion object {
        @JvmStatic fun formatItemPosKey(key: String) : String {
            return "${AppOptions.FIELD_COMMENT_LIST_ITEM_POS}_$key"
        }

        @JvmStatic fun formatItemOffsetKey(key: String) : String {
            return "${AppOptions.FIELD_COMMENT_LIST_ITEM_OFFSET}_$key"
        }

        const val MAX_SAVED_COUNT = 5
    }

    private val dataController: DataController
    private val listView: ListView

    constructor(context: Context,
                key: String,
                dataController: DataController,
                listView: ListView) : this(context, key, 0, dataController, listView)

    constructor(context: Context,
                key: String,
                prependAdapterCount: Int,
                dataController: DataController, listView: ListView) : super(context, key, prependAdapterCount) {
        this.dataController = dataController
        this.listView = listView
    }

    override fun onItemFound(v: View?, visiblePos: Int, position: Int, item: CommentWrapper?) {
        val top = v?.top ?: -1
        saveCommentListItemPosition(key, visiblePos, top)
        Timber.d("key=$key, visiblePos=$visiblePos, top=$top")
    }

    private fun saveCommentListItemPosition(key: String, position: Int, offset: Int) {
        val recentItemList: MutableList<String> = ArrayList()
        val recentItemCsv: String? = dataController.getStringData(AppOptions.FIELD_COMMENT_LIST_ITEM_RECENT)
        recentItemCsv?.let {
            val recentItems: Array<String> = StringUtils.convertCSVToStringArray(it)
            var removedCount = 0

            for (i in 0 until recentItems.size) {
                val recentItemKey = recentItems[i]
                if (recentItems.size - removedCount >= MAX_SAVED_COUNT) {
                    val itemPosKey = formatItemPosKey(recentItemKey)
                    val itemOffsetKey = formatItemOffsetKey(recentItemKey)
                    dataController.remove(itemPosKey)
                    dataController.remove(itemOffsetKey)
                    removedCount++
                    Timber.d("removing itemPosKey=$itemPosKey, itemOffsetKey=$itemOffsetKey")
                } else {
                    recentItemList.add(recentItemKey)
                }
            }
        }

        // To make sure no duplication
        dataController.setIntData(formatItemPosKey(key), position)
        dataController.setIntData(formatItemOffsetKey(key), offset)
        if (!recentItemList.contains(key)) {
            recentItemList.add(key)
            val recent: String = StringUtils.convertToCSVString(recentItemList)
            dataController.setStringData(AppOptions.FIELD_COMMENT_LIST_ITEM_RECENT, recent)
            Timber.d("recentKeys=$recent")
        }
    }

    /**
     * [scrollPosition] An integer array of size 2, which returns the following values
     *
     * 1. Position on where the list should set to
     * 2. Scrolling offset
     *
     */
    fun restoreScrollPosition(scrollPosition: IntArray) {
        val position = dataController.getIntData(formatItemPosKey(key), -1)
        val offset = dataController.getIntData(formatItemOffsetKey(key), -1)

        Timber.d("restoreScrollPosition for key=$key, pos=$position, offset=$offset")
        this.listView.setSelectionFromTop(position + prependAdapterOffset, offset)

        scrollPosition[0] = position + prependAdapterOffset
        scrollPosition[1] = offset
    }
}