package com.under9.android.comments.ui.renderer

import android.content.Context
import android.os.Bundle
import android.view.View
import androidx.dynamicanimation.animation.SpringAnimation
import androidx.dynamicanimation.animation.SpringForce
import androidx.recyclerview.widget.RecyclerView
import com.under9.android.comments.CommentSystem
import com.under9.android.comments.controller.MetricsController
import com.under9.android.comments.data.repository.AppInfoRepositoryInterface
import com.under9.android.comments.event.SublevelLoadMoreEvent
import com.under9.android.comments.integration.DefaultIntegrationModule
import com.under9.android.comments.model.Comment
import com.under9.android.comments.model.wrapper.CommentWrapper
import com.under9.android.comments.model.wrapper.CommentWrapperInterface
import com.under9.android.comments.model.wrapper.CommentWrapperV2
import com.under9.android.comments.ui.widget.CommentItemView
import com.under9.android.commentsystem.R
import kotlinx.android.synthetic.main.comment_system_list_item_v4_buttons_experiment.view.*
import timber.log.Timber

class CommentItemClickListener(
        private val delegate: CommentItemActionDelegate,
        private val appInfoRepository: AppInfoRepositoryInterface,
        private val isShowDownVoteCountEnabled: Boolean,
        private val isReverse: Boolean = false
) : View.OnClickListener {

    override fun onClick(v: View?) {
        v?.let {
            val context: Context = v.context
            val id = v.id
            val comment: CommentWrapperInterface? = v.tag as CommentWrapperInterface
            val viewHolder: RecyclerView.ViewHolder = v.getTag(R.id.commentItemViewHolder) as RecyclerView.ViewHolder
            val position: Int = v.getTag(R.id.commentPosition) as Int

            if (comment == null) return

            with(viewHolder.itemView as CommentItemView) {
                when (id) {
                    R.id.likeBtn,
                    R.id.likeBtnMask,
                    R.id.tvLikeCount -> {
                        Timber.d("parent=${this} ")
                        if (comment.getLikeStatus() == 1) {
                            delegate.unlike(position, comment.getCommentId(), comment)
                        } else {
                            delegate.like(position, comment.getCommentId(), comment)
                        }

                        if (!isShowDownVoteCountEnabled) {
                            // FIXME: 7/1/2019
                        } else {
                            this.setLikeStatus(comment.getLikeCount(), comment.getDislikeCount(), comment.getLikeStatus(), comment.shouldHighlight(), true)
                        }
                    }

                    R.id.dislikeBtn,
                    R.id.dislikeBtnMask -> {
                        if (comment.getLikeStatus() == -1) {
                            delegate.unlike(position, comment.getCommentId(), comment)
                        } else {
                            delegate.dislike(position, comment.getCommentId(), comment)
                        }

                        if (!isShowDownVoteCountEnabled) {
                            // FIXME: 7/1/2019
                        } else {
                            this.setDiskLikeStatus(comment.getLikeCount(), comment.getDislikeCount(), comment.getLikeStatus(), comment.shouldHighlight(), true)
                        }
                    }

                    R.id.replyBtnV4 -> {
                        MetricsController.logUserActions(MetricsController.USER_ACTIONS_REPLY)
                        var prefill: String? = if (comment.isAnonymous()) {
                            "@" + context.getString(R.string.anonymous_mention) + " "
                        } else {
                            if (comment.getDisplayName().isNotEmpty()) "@" + comment.getDisplayName() + " " else  ""
                        }

                        delegate.dispatchOnItemSelectedEvent(position, comment.getId(), comment.getUrl())

                        val suppData = Bundle()
                        suppData.putString(DefaultIntegrationModule.UI_TAP_COMMENT_ID, comment.getCommentId())

                        if (comment.getLevel(appInfoRepository) == appInfoRepository.getMaxCommentLevel()) {
                            val parentComment = comment.getParentComment()
                            if (parentComment != null) {
                                delegate.dispatchOnItemReplyEvent(position, CommentWrapperV2(parentComment), prefill, suppData)
                            } else {
                                delegate.dispatchOnItemReplyEvent(position, comment, prefill, suppData)
                            }
                        }
                        delegate.dispatchOnItemReplyEvent(position, comment, prefill, suppData)
                    }

                    R.id.loadMoreTxt,
                    R.id.loadMoreContainer -> {
                        if (comment is CommentWrapper) {
                            performSublevelLoadMore(comment)
                            return@with
                        }
                    }

                    R.id.dataView -> {

                    }

                    R.id.moreBtn -> {
                        MetricsController.logUserActions(MetricsController.USER_ACTIONS_OPTIONS)
                        delegate.dispatchOnItemMoreEvent(position, comment, comment.getUserName(context))
                    }

                    R.id.userName, R.id.avatar -> {
                        val authorName = v.getTag(R.id.username) as String?
                        authorName?.let { delegate.dispatchOnAuthorNameClickEvent(authorName) }
                    }

                    R.id.avatar -> {
                        val authorName = v.getTag(R.id.username) as String?
                        authorName?.let { delegate.dispatchOnAuthorAvatarClickEvent(authorName) }
                    }

                    R.id.root -> {
                        if (comment.getLevel() <= 1) return
                        if (comment is CommentWrapper) {
                            performSublevelLoadMore(comment)
                            return@with
                        }
                    }
                    R.id.proBadge -> {
                        delegate.dispatchOnItemProBadgeClickEvent(comment)
                    }
                    R.id.refresh -> {
                        delegate.dispatchOnRefreshComment(position, comment)
                    }
                    else -> {
                        Timber.d("Not registered view $v")
                    }
                }
            }

        }
    }

    private fun performSublevelLoadMore(comment: CommentWrapper) {
        comment.isLocalCollapsed = true

        // TODO: 25/2/2019 To be supported
        val firstChild: Comment = if (isReverse) {
            comment.getComment()!!.childrenComments[0]
        } else {
            comment.getComment()!!
        }

        if (comment.rootParentHasNext) {
            comment.isLoadingSublevel = false
            if (comment.parentCommentId != null) {
                CommentSystem.postEvent(comment.getUrl(), SublevelLoadMoreEvent(firstChild.commentId, firstChild.parentCommentId))
            }
            return
        } else {
            CommentSystem.postEvent(comment.getUrl(), SublevelLoadMoreEvent(firstChild.commentId, firstChild.parentCommentId))
        }
    }
}