package com.under9.android.comments.ui.renderer

import android.content.Context
import android.util.TypedValue
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.AbsListView
import androidx.core.widget.TextViewCompat
import androidx.recyclerview.widget.RecyclerView
import com.under9.android.comments.CommentSystem
import com.under9.android.comments.model.wrapper.CommentWrapper
import com.under9.android.comments.model.wrapper.CommentWrapperInterface
import com.under9.android.comments.ui.widget.CommentItemView
import com.under9.android.commentsystem.R
import com.under9.android.lib.util.L10nUtil
import kotlinx.android.synthetic.main.comment_system_list_item_inner_v5.view.*
import timber.log.Timber

class CommentItemRendererV5(
        private val isSeparateSubLevelLoadMore: Boolean,
        private val isReverse: Boolean,
        indentationLevel: Int = 0,
        avatarMode: Int = CommentSystem.AVATAR_MODE_IMAGE,
        shouldHighlightUrlInComment: Boolean = false,
        shouldShowCountry: Boolean = false
) : BaseCommentAdapterRenderer() {
    class ViewHolder(view: View) : RecyclerView.ViewHolder(view)

    private val TAG = "CommentItemRendererV5"
    private val DEBUG = CommentSystem.getInstance().isDebug
    private val commentItemRendererDelegate = CommentItemRendererDelegate(
            appInfoRepository, dataController, commentItemActionDelegate,
            indentationLevel, avatarMode, shouldHighlightUrlInComment, shouldShowCountry
    )

    override fun getView(position: Int,
                         convertView: View?,
                         container: ViewGroup?,
                         item: Any?, renderOptions: RenderOptions?): View {

        var holder: ViewHolder
        var retView: View?
        var wrapper = item as CommentWrapper
        val context = container?.context

        themeAttr.mayInitThemeAttr(context!!)

        if (convertView != null) {
            Timber.d("getView() position=$position , tag=${convertView.tag }, instanceof=${(convertView.tag != null && convertView.tag is ViewHolder)}, isReverse=$isReverse")
            if (convertView.tag == null || convertView.tag !is ViewHolder) {
                retView = null
            }
        }

        if (convertView == null) {
            val inflater = context.getSystemService(Context.LAYOUT_INFLATER_SERVICE) as LayoutInflater
            retView = inflateView(inflater, container)
            holder = ViewHolder(retView)

            retView.setTag(R.id.commentItemViewHolder, holder)
        } else {
            holder = convertView.getTag(R.id.commentItemViewHolder) as ViewHolder
            retView = convertView
        }

        showHideDataView(holder, View.VISIBLE, wrapper)

        commentItemRendererDelegate.applyDrawableStates(holder)
        commentItemRendererDelegate.applyDepth(wrapper, holder)
        commentItemRendererDelegate.applyUserInfo(wrapper, holder, themeAttr)
        commentItemRendererDelegate.applyOpInfo(wrapper, holder, themeAttr)
        commentItemRendererDelegate.applyTextContent(wrapper, holder, themeAttr)
        commentItemRendererDelegate.applyVoteInfo(wrapper, holder, themeAttr)
        commentItemRendererDelegate.applyContentTheme(wrapper, holder, themeAttr)
        commentItemRendererDelegate.applyTextTheme(wrapper, holder, themeAttr)

        // Fix German word-wrapping issue
        TextViewCompat.setAutoSizeTextTypeUniformWithConfiguration(holder.itemView.replyBtnV4,
                12, 14, 1, TypedValue.COMPLEX_UNIT_SP)

        // TODO: 18/2/2019 Non-resuable stuff
//            holder.itemView.loadMoreContainer.visibility = View.GONE

        showHideDataView(holder, View.VISIBLE, wrapper)
        showHideLoadMoreContainer(holder, View.GONE, wrapper)
        showHideBorderAndPlaceholder(holder, View.GONE, wrapper)

        // TODO: 8/2/2019 Hacky fix
        // Related to resetIsCommentThreadEnded() in AbstractCommentSystemListingFragment
        // after load more
        if (isSeparateSubLevelLoadMore) {
            if (wrapper.getLevel() > 1 && !wrapper.shouldHighlight()) {
                showHideDataView(holder, View.GONE, wrapper)
            }
        }


        // TODO: 1/3/2019 Need to think about how to make it better,
        // May be displaying by view type?
        if (wrapper.numOfRootParentsChildren > 0) {
            if (wrapper.isCommentThreadEnded) {
                if (wrapper.isLocalCollapsed) {
                    showHideDataView(holder, View.GONE, wrapper)
                    showHideLoadMoreContainer(holder, View.VISIBLE, wrapper)
                    showHideBorderAndPlaceholder(holder, View.VISIBLE, wrapper)

                    holder.itemView.loadMoreTxt.text = L10nUtil.getFormattedQuantityString(holder.itemView.context, R.plurals.view_n_replies, wrapper.rootParent.childrenTotal)
                } else {
                    if (wrapper.rootParentHasNext) {
                        showHideLoadMoreContainer(holder, View.VISIBLE, wrapper)
                        showHideBorderAndPlaceholder(holder, View.VISIBLE, wrapper)

                        holder.itemView.loadMoreTxt.text = L10nUtil.getFormattedQuantityString(holder.itemView.context, R.plurals.view_n_replies, wrapper.rootParent.childrenTotal)
                    } else {
                        showHideLoadMoreContainer(holder, View.GONE, wrapper)
                        showHideBorderAndPlaceholder(holder, View.VISIBLE, wrapper)
                    }
                }
            }
        } else {
            if (isReverse) {
                val childrenTotal = wrapper.getComment()?.childrenTotal ?: 0
                if (childrenTotal > 0) {
                    showHideLoadMoreContainer(holder, View.VISIBLE, wrapper)
                    holder.itemView.loadMoreTxt.text = L10nUtil.getFormattedQuantityString(holder.itemView.context, R.plurals.view_n_replies, childrenTotal)
                    holder.itemView.bottomPlaceholder.visibility = View.VISIBLE
                } else {
                    showHideLoadMoreContainer(holder, View.GONE, wrapper)
                    holder.itemView.bottomPlaceholder.visibility = View.GONE
                }

                holder.itemView.commentBorder.visibility = View.VISIBLE

            } else {
                showHideLoadMoreContainer(holder, View.GONE, wrapper)
                if (wrapper.isCommentThreadEnded) {
                    showHideBorderAndPlaceholder(holder, View.VISIBLE, wrapper)
                    holder.itemView.bottomPlaceholder.visibility = View.GONE
                } else {
                    showHideBorderAndPlaceholder(holder, View.GONE, wrapper)
                }
            }
        }

        if (isReverse) {
            holder.itemView.refresh.visibility = if (position == count - 1) View.VISIBLE else View.GONE
        }

        Timber.d("~ comment, fillContent " + wrapper.getUserName(holder.itemView.context) + " " + wrapper.getCommentId()
                + " content=" + wrapper.getContent() + ", textContent = ${commentItemRendererDelegate.getTextContent(wrapper)}\n"
                //                + " parentCommentListItem: " + ((parentCommentListItem == null) ? null : parentCommentListItem.getComment().getCommentText())+ "\n"
                + ", isLast=" + wrapper.isCommentThreadEnded + "\n"
                + ", currComment=" + wrapper.getComment() + "\n"
                + ", numOfRootParentsChildren=" + wrapper.numOfRootParentsChildren + "\n"
                + ", rootParentHasNext=" + wrapper.rootParentHasNext + "\n"
                + ", isLocalCollapsed=" + wrapper.isLocalCollapsed + "\n"
                + ", pos=" + position + "\n"
                + ", parentId=" + wrapper.parentId
                + ", child=" + wrapper.getComment()!!.childrenComments
                + ", childrenTotal=" + wrapper.getComment()!!.childrenTotal

        )

        commentItemRendererDelegate.bindListener(wrapper, retView, holder, position)

        // FIXME: 11/4/2019 Hacky fix, when in reverse layout, "View more replies" button
        // is stick to the same wrapper itself, therefore we need to use the first child as
        // load more offset, to make sure the behaviour is the same as normal ordering
        if (isReverse) {
            if (wrapper.getComment()?.childrenTotal ?: 0 > 0 && wrapper.getComment()?.childrenComments?.size ?: 0 > 0) {
                holder.itemView.loadMoreTxt.tag = CommentWrapper(wrapper.getComment()!!.childrenComments[0])
            }
        }

        return retView
    }


    open protected fun inflateView(layoutInflater: LayoutInflater, container: ViewGroup?) : View {
        val v = CommentItemView(container!!.context)
        v.layoutParams = AbsListView.LayoutParams(AbsListView.LayoutParams.MATCH_PARENT, AbsListView.LayoutParams.WRAP_CONTENT)
        return v
    }

    private fun showHideDataView(holder: ViewHolder, visibility: Int, wrapper: CommentWrapperInterface) {
        with(holder.itemView) {
            avatar.visibility = visibility
//            avatarPlaceholder.visibility = visibility
            userName.visibility = visibility
            meta.visibility = visibility
            content.visibility = if (content.text.isEmpty()) View.GONE else visibility

            if(wrapper.getIsProUser() || wrapper.getIsProPlusUser()){
                proBadge.visibility = visibility
            } else {
                proBadge.visibility = View.GONE
            }
            replyBtnV4.visibility = visibility
            likeBtn.visibility = visibility
            dislikeBtn.visibility = visibility
            moreBtn.visibility = visibility
        }
    }

    private fun showHideLoadMoreContainer(holder: ViewHolder, visibility: Int, wrapper: CommentWrapperInterface) {
        with(holder.itemView) {
            when (visibility) {
                View.VISIBLE -> {
                    if (wrapper is CommentWrapper) {
                        if (wrapper.isLoadingSublevel) {
                            loadMoreTxt.visibility = View.GONE
                            progress.visibility = View.VISIBLE
                        } else {
                            loadMoreTxt.visibility = View.VISIBLE
                            progress.visibility = View.GONE
                        }
                    } else {
                        loadMoreTxt.visibility = View.VISIBLE
                        progress.visibility = View.GONE
                    }
                }
                View.GONE -> {
                    loadMoreTxt.visibility = View.GONE
                    progress.visibility = View.GONE
                }
            }
        }
    }

    private fun showHideBorderAndPlaceholder(holder: ViewHolder, visibility: Int, wrapper: CommentWrapperInterface) {

        with(holder.itemView) {
            commentBorder.visibility = visibility
            bottomPlaceholder.visibility = visibility
        }
    }
}