package com.under9.android.comments.ui.renderer

import android.content.Context
import androidx.recyclerview.widget.RecyclerView
import com.under9.android.comments.controller.DataController
import com.under9.android.comments.data.repository.AppInfoRepositoryInterface
import com.under9.android.comments.model.constant.CommentConstant
import com.under9.android.comments.model.wrapper.CommentWrapperInterface
import com.under9.android.comments.model.wrapper.EmbedImage
import com.under9.android.commentsystem.R
import com.under9.android.lib.network.UrlMapperInterface
import com.under9.android.lib.widget.uiv3.UniversalImageView
import com.under9.android.lib.widget.uiv3.adapter.ImageTileInfoAdapter
import com.under9.android.lib.widget.uiv3.adapter.UIVAdapter
import com.under9.android.lib.widget.uiv3.adapter.VideoInfoAdapter
import kotlinx.android.synthetic.main.comment_system_list_item_inner_v5.view.*

class CommentMediaItemRendererDelegate(
        private val appInfoRepository: AppInfoRepositoryInterface,
        private val dataController: DataController,
        private val dmm: UrlMapperInterface,
        private val commentItemActionDelegate: CommentItemActionDelegate) {

    private val imageOnClickListener = UniversalImageView.OnClickListener { view, adapter, uiv ->
        val wrapper = uiv.getTag(R.id.comment_wrapper) as CommentWrapperInterface
        commentItemActionDelegate.dispatchOnItemOpenOverlay(wrapper, adapter)
    }

    private val imageOnLongClickListener = UniversalImageView.OnLongClickListener { view, _, uiv ->
        val wrapper = uiv.getTag(R.id.comment_wrapper) as CommentWrapperInterface
        commentItemActionDelegate.dispatchOnMediaItemLongClickEvent(view.getTag(R.id.commentPosition) as? Int ?: 0, wrapper)
    }

    private fun getEmbedImage(commentWrapper: CommentWrapperInterface): EmbedImage? {
        return if (CommentConstant.TYPE_USER_MEDIA == commentWrapper.getType()) {
            commentWrapper.getFirstMediaEmbedImage()
        } else {
            commentWrapper.getEmbedMediaMeta()?.embedImage
        }
    }

    fun bindMediaContent(holder: RecyclerView.ViewHolder,
                         position: Int, wrapper: CommentWrapperInterface,
                         context: Context) {

        with(holder.itemView) {
            image.setTag(R.id.image_holder, holder)
            image.setTag(R.id.comment_wrapper, wrapper)

            val embedImage: EmbedImage? = getEmbedImage(wrapper)
            val mediaType: String = embedImage?.type ?: ""

            if (embedImage == null) return

            val prevItem = image.getTag(R.id.commentItemViewHolder)

            if (prevItem == wrapper) {
                holder.itemView.requestLayout()
                return
            }

            when (mediaType) {
                CommentConstant.MEDIA_TYPE_STATIC -> {
                    embedImage.image?.let {
                        val height = it.height
                        val width = it.width
                        val url = it.url ?: ""

                        val imageTileInfoAdapter = ImageTileInfoAdapter.newBuilder()
                                .setImage(url, width, height)
                                .build()

                        val adapter = UIVAdapter.newBuilder(UniversalImageView.MODE_STATIC_IMAGE)
                                .setUrlMapper(dmm)
                                .setRetryDrawable(R.drawable.icn_retry)
                                .setImageTileInfoAdapter(imageTileInfoAdapter)
                                .setOnClickListener(imageOnClickListener)
                                .setOnLongClickListener(imageOnLongClickListener)
                                .build()
                        image.adapter = adapter
                    }
                }
                CommentConstant.MEDIA_TYPE_ANIMATED -> {
                    val embedVideo = embedImage.video
                    val embedIMage = embedImage.image
                    if (embedVideo != null && embedIMage != null) {
                        val url = embedVideo.url

                        val height = embedIMage.height

                        val videoInfoAdapter = VideoInfoAdapter.newBuilder()
                                .videoUrl(url)
                                .build()

                        val imageTileInfoAdapter = ImageTileInfoAdapter.newBuilder()
                                .setImage(embedIMage.url, embedIMage.width, height)
                                .build()

                        val adapter = UIVAdapter.newBuilder(UniversalImageView.MODE_MP4_WITHOUT_SOUND)
                                .setRetryDrawable(R.drawable.icn_retry)
                                .centerBadge(R.drawable.ic_play_gif)
                                .setUrlMapper(dmm)
                                .setImageTileInfoAdapter(imageTileInfoAdapter)
                                .setVideoInfoAdapter(videoInfoAdapter)
                                .setOnLongClickListener(imageOnLongClickListener)
                                .build()

                        image.adapter = adapter
                    } else {
                        return
                    }

                }
                else -> {}
            }
        }

    }
}