package com.under9.android.comments.ui.renderer

import android.content.Context
import android.view.View
import com.under9.android.comments.CommentSystem
import com.under9.android.comments.controller.DataController
import com.under9.android.comments.data.repository.AppInfoRepositoryInterface
import com.under9.android.comments.model.ui.CommentItemThemeAttr
import com.under9.android.comments.model.wrapper.CommentWrapperV2
import com.under9.android.comments.ui.widget.CommentItemView
import com.xwray.groupie.ViewHolder
//import kotlinx.android.synthetic.main.comment_system_list_item_inner_v5.view.*

class DefaultCommentItemItemBinder(
        private val dataController: DataController,
        private val appInfoRepository: AppInfoRepositoryInterface,
        private val commentItemActionDelegate: CommentItemActionDelegate,
        private val commentItemClickListener: CommentItemClickListener
) : ExpandableCommentItemBinder<ViewHolder> {

    companion object {
        val DEBUG = CommentSystem.getInstance().isDebug
        const val TAG = "DefaultCommentItemItemBinder"
        const val TEXT_OP_LABEL = "OP"
        const val MEMEFUL_URL = "memeful.com"
    }

    private val indentationLevel: Int = 0
    private val commentItemRendererDelegate = CommentItemRendererDelegate(
            appInfoRepository, dataController, commentItemActionDelegate, indentationLevel
    )

    override fun bind(commentWrapper: CommentWrapperV2, depth: Int,
                      themeAttr: CommentItemThemeAttr, viewHolder: ViewHolder, position: Int) {
        val context: Context = viewHolder.itemView.context
        themeAttr.mayInitThemeAttr(context)

        with(viewHolder.itemView as CommentItemView) {
            loadMoreTxt.visibility = View.GONE
            progressBar.visibility = View.GONE
            uiv.visibility = View.GONE
        }

        commentItemRendererDelegate.applyDrawableStates(viewHolder)
        commentItemRendererDelegate.applyDepth(commentWrapper, viewHolder)
        commentItemRendererDelegate.applyUserInfo(commentWrapper, viewHolder, themeAttr)
        commentItemRendererDelegate.applyOpInfo(commentWrapper, viewHolder, themeAttr)
        commentItemRendererDelegate.applyTextContent(commentWrapper, viewHolder, themeAttr)
        commentItemRendererDelegate.applyVoteInfo(commentWrapper, viewHolder, themeAttr)
        commentItemRendererDelegate.applyContentTheme(commentWrapper, viewHolder, themeAttr)
        commentItemRendererDelegate.applyTextTheme(commentWrapper, viewHolder, themeAttr)
        commentItemRendererDelegate.bindListener(commentWrapper, viewHolder.itemView, viewHolder, position)

        (viewHolder.itemView as CommentItemView).meta.visibility = View.VISIBLE

    }
}