package com.under9.android.comments.ui.renderer

import com.under9.android.comments.controller.DataController
import com.under9.android.comments.data.repository.AppInfoRepositoryInterface
import com.under9.android.comments.model.constant.CommentConstant
import com.under9.android.comments.model.ui.CommentItemThemeAttr
import com.under9.android.comments.model.wrapper.CommentWrapperV2
import com.under9.android.comments.ui.fragment.singlecomment.ExpandableCommentItemBinderProvider
import com.under9.android.commentsystem.R
import com.under9.android.lib.network.UrlMapperInterface
import com.xwray.groupie.ExpandableGroup
import com.xwray.groupie.ExpandableItem
import com.xwray.groupie.Item
import com.xwray.groupie.ViewHolder

/**
 * Binder interface for rendering comment item
 */
interface ExpandableCommentItemBinder<VH : ViewHolder> {
    fun bind(commentWrapper: CommentWrapperV2,
             depth: Int,
             themeAttr: CommentItemThemeAttr,
             viewHolder: VH,
             position: Int)
}

internal open class ExpandableCommentItem(
        private val commentWrapper: CommentWrapperV2,
        private val depth: Int,
        private val dataController: DataController,
        private val appInfoRepository: AppInfoRepositoryInterface,
        private val urlMapperInterface: UrlMapperInterface,
        private val commentItemThemeAttr: CommentItemThemeAttr,
        private val commentItemActionDelegate: CommentItemActionDelegate,
        private val expandableCommentItemBinderProvider: ExpandableCommentItemBinderProvider
) : Item<ViewHolder>(), ExpandableItem {

    private lateinit var expandableGroup: ExpandableGroup

    override fun getLayout(): Int {
        return R.layout.comment_system_list_item_v5
        // FIXME: 3/1/2019 Going to roll out this experiment soon
    }

    override fun bind(viewHolder: ViewHolder, position: Int) {
        // Bind by default
        expandableCommentItemBinderProvider
                .obtainBinder(dataController, appInfoRepository, urlMapperInterface, commentItemActionDelegate)
                .bind(commentWrapper, depth, commentItemThemeAttr, viewHolder, position)
        expandableCommentItemBinderProvider
                .obtainBinder(dataController, appInfoRepository, urlMapperInterface, commentItemActionDelegate, commentWrapper.getType())
                .bind(commentWrapper, depth, commentItemThemeAttr, viewHolder, position)
    }

    override fun setExpandableGroup(onToggleListener: ExpandableGroup) {
        this.expandableGroup = onToggleListener
    }

    fun getCommentWrapper(): CommentWrapperV2 {
        return this.commentWrapper
    }
}