package com.under9.android.lib.bottomsheet

import android.content.Context
import android.graphics.PorterDuff
import android.os.Build
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.ImageView
import androidx.appcompat.content.res.AppCompatResources
import androidx.core.content.ContextCompat
import androidx.core.graphics.drawable.DrawableCompat
import androidx.recyclerview.widget.RecyclerView
import com.under9.android.lib.util.UiUtil
import com.under9.android.lib.widget.R
import kotlinx.android.synthetic.main.view_item_simple_bottom_sheet.view.*


class BottomSheetAdapter(private val titles: List<String>,
                         private val icons: List<Int>,
                         private val ids: List<Int>,
                         private val selectedIndex: Int = -1,
                         private val clickListener: BottomSheetClickListener,
                         private val dismissListener: DismissListener): RecyclerView.Adapter<BottomSheetAdapter.ViewHolder>() {

    override fun onCreateViewHolder(parent: ViewGroup, viewType: Int) = ViewHolder(LayoutInflater.from(parent.context).inflate(R.layout.view_item_simple_bottom_sheet, parent, false))

    override fun getItemCount() = titles.size

    override fun onBindViewHolder(holder: ViewHolder, position: Int) {
        holder.bind(titles[position], icons[position], position, ids[position])
    }

    inner class ViewHolder(itemView: View): RecyclerView.ViewHolder(itemView) {

        fun bind(title: String, resIcon: Int, pos: Position, id: MenuItemId) {
            with(itemView) {
                menuTitle.text = title

                // Currently, just apply the change of dark theme for the version >= LOLLIPOP, because it make codes very messy if handle pre-lollipop here,
                // And it will drop pre-lollipop later, so TODO: clean up when dropped pre-lollipop support
                if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP) {
                    itemView.setBackgroundColor(UiUtil.themeAttributeToColor(R.attr.under9_themeForeground, context, -1))
                    menuTitle.setTextColor(UiUtil.themeAttributeToColor(R.attr.under9_themeTextColorPrimary, context, -1))
                }
                applyIconTintColor(menuIcon, resIcon, itemView.context)
                setOnClickListener {
                    clickListener(pos, id)
                    dismissListener()
                }

                if (pos == selectedIndex) {
                    menuChecked.visibility = View.VISIBLE
                } else {
                    menuChecked.visibility = View.GONE
                }
            }
        }

        private fun applyIconTintColor(menuIcon: ImageView, resIcon: Int, context: Context) {
            if (Build.VERSION.SDK_INT < Build.VERSION_CODES.LOLLIPOP) {
                val drawable = AppCompatResources.getDrawable(context, resIcon)
                val wrapDrawable = DrawableCompat.wrap(drawable!!).also {
                    DrawableCompat.setTint(it, UiUtil.themeAttributeToColor(R.attr.under9_themeIconColor, context, -1))
                    DrawableCompat.setTintMode(it, PorterDuff.Mode.SRC_IN)
                }
                menuIcon.background = wrapDrawable
            } else {
                menuIcon.background = ContextCompat.getDrawable(itemView.context, resIcon)
            }
        }
    }
}