package com.under9.android.lib.bottomsheet

import android.app.Dialog
import android.os.Build
import android.os.Bundle
import android.view.ViewGroup
import android.widget.FrameLayout
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.recyclerview.widget.RecyclerView
import com.under9.android.lib.bottomsheet.data.BottomSheetMenuItems
import com.under9.android.lib.util.UiUtil
import com.under9.android.lib.widget.R

class GagBottomSheetDialogFragment: StyledBottomSheetDialogFragment() {

    private lateinit var titles: List<String>
    private lateinit var icons: List<Int>
    private lateinit var ids: List<Int>
    private var selectedIndex: Int = -1
    private var listener: BottomSheetClickListener = { _: Position, _: MenuItemId -> }

    companion object {
        private const val KEY_MENU_ITEMS = "menu_items"

        const val FRAG_TAG = "more_action"

        @JvmStatic
        fun newInstance(items: BottomSheetMenuItems): GagBottomSheetDialogFragment {
            val args = Bundle()
            args.putParcelable(KEY_MENU_ITEMS, items)
            val frag = GagBottomSheetDialogFragment()
            frag.arguments = args
            return frag
        }
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        val items = arguments?.getParcelable<BottomSheetMenuItems>(KEY_MENU_ITEMS)
        items?.titles?.let { titles = it }
        items?.icons?.let { icons = it }
        items?.ids?.let { ids = it }
        selectedIndex = items?.selectedIndex ?: -1
    }

    fun setBottomSheetClickListener(listener: BottomSheetClickListener){
        this.listener = listener
    }

    override fun onCreateDialog(savedInstanceState: Bundle?): Dialog {
        val dialog = super.onCreateDialog(savedInstanceState)
        val matchAllParentParam = FrameLayout.LayoutParams(ViewGroup.LayoutParams(FrameLayout.LayoutParams.MATCH_PARENT, FrameLayout.LayoutParams.MATCH_PARENT))

        val container = FrameLayout(context!!).apply {
            // Currently, just apply the change of dark theme for the version >= LOLLIPOP, because it make codes very messy if handle pre-lollipop here,
            // And it will drop pre-lollipop later, so TODO: clean up when dropped pre-lollipop support
            if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP) {
                setBackgroundColor(UiUtil.themeAttributeToColor(R.attr.under9_themeForeground, context, -1))
            }
            layoutParams = matchAllParentParam
            id = R.id.more_menu_button_sheet
        }

        val rv = RecyclerView(context!!).apply {
            layoutParams =  FrameLayout.LayoutParams(ViewGroup.LayoutParams(FrameLayout.LayoutParams.MATCH_PARENT, FrameLayout.LayoutParams.MATCH_PARENT)).apply {
                bottomMargin = context!!.resources.getDimension(R.dimen.space8).toInt()
            }
            id = R.id.more_menu_recyclerview
            layoutManager = LinearLayoutManager(context)
            adapter = BottomSheetAdapter(titles, icons, ids, selectedIndex, listener, {
                dialog.dismiss()
            })
            hasFixedSize()
        }

        container.addView(rv)
        dialog.setContentView(container)
        return dialog
    }
}

typealias BottomSheetClickListener = ((Position), (MenuItemId)) -> Unit
typealias DismissListener = () -> Unit
typealias MenuItemId = Int
typealias Position = Int