package com.under9.android.lib.bottomsheet

import android.app.Dialog
import android.os.Build
import android.os.Bundle
import android.view.View
import androidx.core.content.ContextCompat
import com.google.android.material.bottomsheet.BottomSheetBehavior
import com.google.android.material.bottomsheet.BottomSheetDialog
import com.google.android.material.bottomsheet.BottomSheetDialogFragment
import com.under9.android.lib.util.UiUtil
import com.under9.android.lib.widget.R



/**
 * Round corner bottom sheet, should just define style here
 * */
abstract class StyledBottomSheetDialogFragment(bottomSheet: View? = null): BottomSheetDialogFragment(){

    private var bottomSheetBehavior: BottomSheetBehavior<View>? = null

    private var behaviorCallback: BottomSheetBehavior.BottomSheetCallback = object : BottomSheetBehavior.BottomSheetCallback() {
        override fun onStateChanged(bottomSheet: View, newState: Int) {
            if (newState == BottomSheetBehavior.STATE_HIDDEN) {
                dismiss()
            }
        }

        override fun onSlide(bottomSheet: View, slideOffset: Float) {
        }
    }

    init {
        if (bottomSheet != null) {
            bottomSheetBehavior = BottomSheetBehavior.from<View>(bottomSheet)
            bottomSheetBehavior!!.setBottomSheetCallback(behaviorCallback)
        }
    }

    override fun getTheme() = R.style.BottomSheetDialogTheme

    override fun onCreateDialog(savedInstanceState: Bundle?): Dialog {
        val dialog = BottomSheetDialog(requireContext(), theme)
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP) {
            if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O_MR1) {
                // Manually apply the navigation bar color because of:
                // https://stackoverflow.com/questions/48785266/oneplus-5t-on-oxygen-os-ignores-the-specified-color-of-the-navigation-bar
                dialog.window?.navigationBarColor = UiUtil.themeAttributeToColor(R.attr.under9_themeNavigationBarColor, context, -1)
            } else {
                dialog.window?.navigationBarColor = ContextCompat.getColor(context!!, R.color.under9_theme_black)
            }
        }
        return dialog
    }

    fun setBehaviorCallback(callback: BottomSheetBehavior.BottomSheetCallback) {
        this.behaviorCallback = callback
        bottomSheetBehavior?.setBottomSheetCallback(callback)
    }
}