package com.under9.android.lib.core.mvvm

import android.app.Application
import androidx.lifecycle.AndroidViewModel
import io.reactivex.disposables.CompositeDisposable
import io.reactivex.disposables.Disposable
import timber.log.Timber

abstract class RxAndroidViewModel(application: Application) : AndroidViewModel(application) {
    val disposables = CompositeDisposable()

    protected fun <T : Disposable> T.addSubscription(): T {
        disposables.add(this)
        return this
    }

    override fun onCleared() {
        super.onCleared()
        disposables.dispose()
        Timber.d("VM cleared")
    }
}