package com.under9.android.lib.util

import android.content.Context
import android.content.res.ColorStateList
import android.graphics.PorterDuff
import android.graphics.drawable.Drawable
import androidx.annotation.*
import androidx.annotation.IntRange
import androidx.appcompat.content.res.AppCompatResources
import androidx.core.graphics.drawable.DrawableCompat

object ColorUtils {
    /**
     *
     */
    fun createCheckableStateList(@ColorInt checkedColor: Int,
                                 @ColorInt uncheckedColor: Int): ColorStateList {
        val states = arrayOf(
                intArrayOf(android.R.attr.state_checked),
                intArrayOf(-android.R.attr.state_checked)
        )
        val colors = intArrayOf(checkedColor, uncheckedColor)
        return ColorStateList(states, colors)
    }

    /**
     *
     */
    fun getTintedVectorDrawable(context: Context,
                             @DrawableRes drawableRes: Int,
                             @ColorInt tint: Int): Drawable? {
        val d = AppCompatResources.getDrawable(context, drawableRes) ?: return null
        val wrappedDrawable = DrawableCompat.wrap(d)
        DrawableCompat.setTint(wrappedDrawable, tint)
        DrawableCompat.setTintMode(wrappedDrawable, PorterDuff.Mode.SRC_IN)
        return wrappedDrawable
    }

    /**
     * Tinting a vector drawable with state
     */
    fun getTintedVectorDrawable(context: Context,
                             @DrawableRes drawableRes: Int,
                             colorStateList: ColorStateList?): Drawable? {
        val d = AppCompatResources.getDrawable(context, drawableRes) ?: return null
        val wrappedDrawable = DrawableCompat.wrap(d)
        DrawableCompat.setTintList(wrappedDrawable, colorStateList)
        DrawableCompat.setTintMode(wrappedDrawable, PorterDuff.Mode.SRC_IN)
        return wrappedDrawable
    }

    /**
     *
     */
    fun getTintedDrawable(drawable: Drawable, @ColorInt color: Int): Drawable? {
        val newDrawable = DrawableCompat.wrap(drawable).mutate()
        DrawableCompat.setTint(newDrawable, color)
        DrawableCompat.setTintMode(newDrawable, PorterDuff.Mode.SRC_IN)
        return newDrawable
    }

    // Copied from the Plaid app
    /**
     * Set the alpha component of `color` to be `alpha`.
     */
    @CheckResult
    @ColorInt
    fun modifyAlpha(@ColorInt color: Int,
                    @IntRange(from = 0, to = 255) alpha: Int): Int {
        return color and 0x00ffffff or (alpha shl 24)
    }

    /**
     * Set the alpha component of `color` to be `alpha`.
     */
    @CheckResult
    @ColorInt
    fun modifyAlpha(@ColorInt color: Int,
                    @FloatRange(from = 0.0, to = 1.0) alpha: Float): Int {
        return modifyAlpha(color, (255f * alpha).toInt())
    }

    @JvmStatic
    fun isDark(@ColorInt color: Int): Boolean {
        return androidx.core.graphics.ColorUtils.calculateLuminance(color) < 0.5
    }
}