package com.under9.android.lib.util

import android.view.View
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.recyclerview.widget.RecyclerView
import timber.log.Timber
import java.lang.ref.WeakReference

/**
 * Created by raymond on 8/8/2017.
 */

class RecyclerViewListItemFinder<ITEM>(recyclerView: RecyclerView,
                                       private var itemList: List<ITEM>) : IListItemFinder<ITEM> {

    private val mRef: WeakReference<RecyclerView> = WeakReference(recyclerView)

    fun updateItemList(itemList: List<ITEM>) {
        this.itemList = itemList
    }

    override fun findListItem(finderHelper: ListItemFindingStrategy<ITEM>) {
        val recyclerView = mRef.get() ?: return

        if (recyclerView.layoutManager !is LinearLayoutManager) return

        val linearLayoutManager = recyclerView.layoutManager as LinearLayoutManager? ?: return
        val first = linearLayoutManager.findFirstVisibleItemPosition()
        Timber.d("checkAutoPlay firstVisible:$first")
        Timber.d("checkAutoPlay fv:"
                + linearLayoutManager.findFirstVisibleItemPosition()
                + " fcvp:" + linearLayoutManager.findFirstCompletelyVisibleItemPosition()
                + " lvp:" + linearLayoutManager.findLastVisibleItemPosition()
                + " lcvp:" + linearLayoutManager.findLastCompletelyVisibleItemPosition())

        val count = linearLayoutManager.childCount

        val upperLine = finderHelper.upperLine
        val lowerLine = finderHelper.lowerLine
        val listSize = itemList.size
        var view: View? = null

        var foundPos = NOT_FOUND_POS
        for (i in 0 until count) {
            val pos = first + i - finderHelper.prependAdapterOffset

            if (!finderHelper.shouldCheckView(linearLayoutManager.getChildAt(i), pos)) {
                continue
            }
            view = linearLayoutManager.getChildAt(i)
            val top = view!!.top
            val bottom = view.bottom
            if (top <= upperLine && bottom >= lowerLine) {
                foundPos = pos
                if (foundPos > NOT_FOUND_POS && foundPos < listSize) {
                    finderHelper.onItemFound(view, first, foundPos, itemList[foundPos])
                }
            }
            if (top > lowerLine || bottom < upperLine) {
                finderHelper.onViewPassed(view, first, pos)
            }
        }


        if (itemList.isEmpty()) {
            finderHelper.onItemPositionOutOfBound(view, first, NOT_FOUND_POS)
        } else {
            if (foundPos >= listSize) {
                finderHelper.onItemPositionOutOfBound(view, first, foundPos)
            }
        }
    }

    companion object {
        private val TAG = "RecyclerViewListItemFinder"
        private val DEBUG = false

        private val NOT_FOUND_POS = -1
    }
}

