package com.under9.android.lib.util

import android.content.Context
import com.under9.android.lib.common.R

class TimeAgo(context: Context): TimeFormatter {
    private val sMinutes: String = context.resources.getString(R.string.minutes)
    private val sHours: String = context.resources.getString(R.string.hours)
    private val sDays: String = context.resources.getString(R.string.days)
    private val sWeeks: String = context.resources.getString(R.string.weeks)
    private val sMonths: String = context.resources.getString(R.string.months)
    private val sYears: String = context.resources.getString(R.string.years)
    private val sJustNow: String = context.resources.getString(R.string.time_ago_just_now)

    override fun timeAgo(millis: Long): String {
        return formatTime(millis)
    }

    override fun formatTime(millis: Long): String {
        val diff = System.currentTimeMillis() - millis

        val seconds = (Math.abs(diff) / 1000).toInt()
        val minutes = seconds / 60
        val hours = minutes / 60
        val days = hours / 24
        val weeks = days / 7
        val months = days / 30
        val years = days / 365

        return when {
            seconds < 60 -> sJustNow
            minutes < 60 -> minutes.toString() + sMinutes
            hours < 24 -> hours.toString() + sHours
            days < 7 -> days.toString() + sDays
            weeks < 6 -> weeks.toString() + sWeeks
            years < 1 -> months.toString() + sMonths
            else -> years.toString() + sYears
        }
    }

}
