package com.under9.android.lib.util

import android.annotation.TargetApi
import android.graphics.Rect
import android.os.Build
import android.os.Looper
import android.view.Menu
import android.view.View
import android.view.Window
import androidx.annotation.ColorInt
import androidx.appcompat.widget.Toolbar

object ViewUtils {

    @JvmStatic
    fun runOnUiThread(view: View, runnable: Runnable) {
        if (Looper.myLooper() == Looper.getMainLooper()) {
            runnable.run()
        } else {
            view.post(runnable)
        }
    }

    /**
     * Bug for [before 4.3][View.getHitRect]
     * @see {http://stackoverflow.com/questions/17750116/strange-view-gethitrect-behaviour}
     *
     * @param v
     * @param rect
     */
    @TargetApi(Build.VERSION_CODES.HONEYCOMB)
    @JvmStatic
    fun getHitRectCompat(v: View, rect: Rect) {
        rect.left = (v.left + v.translationX).toInt()
        rect.top = (v.top + v.translationY).toInt()
        rect.right = rect.left + v.width
        rect.bottom = rect.top + v.height
    }

    @JvmStatic
    fun lightStatusBar(view: View) {
        if (Build.VERSION.SDK_INT >= 23) {
            var flags = view.systemUiVisibility
            flags = flags or View.SYSTEM_UI_FLAG_LIGHT_STATUS_BAR
            view.systemUiVisibility = flags
        }
    }

    @JvmStatic
    fun clearLightStatusBar(view: View) {
        if (Build.VERSION.SDK_INT >= 23) {
            var flags = view.systemUiVisibility
            flags = flags and View.SYSTEM_UI_FLAG_LIGHT_STATUS_BAR.inv()
            view.systemUiVisibility = flags
        }
    }

    @JvmStatic
    fun updateToolbarColor(toolbar: Toolbar, @ColorInt color: Int) {
        updateToolbarColor(toolbar.menu, color)
    }

    @JvmStatic
    fun updateToolbarColor(menu: Menu, @ColorInt color: Int) {
        for (i in 0 until menu.size()) {
            val menuItem = menu.getItem(i)
            val icon = menuItem.icon
            val newIcon = ColorUtils.getTintedDrawable(icon, color)
            menu.getItem(i).icon = newIcon
        }
    }
}
