package com.under9.android.lib.widget

import android.content.Context
import android.graphics.drawable.Drawable
import android.net.Uri
import android.util.AttributeSet
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup

import android.widget.FrameLayout
import com.facebook.drawee.drawable.ScalingUtils
import com.facebook.drawee.generic.RoundingParams
import com.facebook.drawee.view.SimpleDraweeView
import com.under9.android.lib.util.UiUtil
import timber.log.Timber

class ActiveAvatarView : FrameLayout {
    var isActive: Boolean = false
        set(value) {
            field = value
            activeBadge.visibility = if (value) View.VISIBLE else View.GONE
        }

    lateinit var activeBadge: View
    lateinit var avatar: SimpleDraweeView

    constructor(context: Context): this(context, null)

    constructor(context: Context, attributeSet: AttributeSet?): this(context, attributeSet, 0)

    constructor(context: Context, attributeSet: AttributeSet?, defStyle: Int): super(context, attributeSet, defStyle) {
        attributeSet?.let {
            loadAttributes(context, attributeSet, defStyle)
        }
    }

    private fun loadAttributes(context: Context, attrs: AttributeSet, deftStyleAttr: Int) {
        val a = context.theme.obtainStyledAttributes(attrs, R.styleable.ActiveAvatarView, deftStyleAttr, deftStyleAttr)
        try {
            LayoutInflater.from(context).inflate(R.layout.view_active_avatar_small, this, true)
            avatar = findViewById(R.id.avatarImage)
            activeBadge = findViewById(R.id.activeBadge)

            val hierarchy = avatar.hierarchy
            if (a.getResourceId(R.styleable.ActiveAvatarView_placeholderImage, 0) != 0) {
                hierarchy.setPlaceholderImage(
                        a.getResourceId(R.styleable.ActiveAvatarView_placeholderImage, 0),
                        ScalingUtils.ScaleType.FIT_CENTER
                )
            }

            if (a.getResourceId(R.styleable.ActiveAvatarView_failureImage, 0) != 0) {
                hierarchy.setFailureImage(
                        a.getResourceId(R.styleable.ActiveAvatarView_failureImage, 0),
                        ScalingUtils.ScaleType.FIT_CENTER
                )
            }

            if (a.getBoolean(R.styleable.SimpleDraweeView_roundAsCircle, false)) {
                hierarchy.roundingParams = RoundingParams.asCircle()
            }

            val defaultHeight = UiUtil.dpToPx(context, 12)
            val defaultWidth = UiUtil.dpToPx(context, 12)
            val defaultMargin = UiUtil.dpToPx(context, 1)

            val badgeTopMargin = a.getDimensionPixelSize(R.styleable.ActiveAvatarView_badgeMarginTop, defaultMargin)
            val badgeBottomMargin = a.getDimensionPixelSize(R.styleable.ActiveAvatarView_badgeMarginBottom, defaultMargin)
            val badgeStartMargin = a.getDimensionPixelSize(R.styleable.ActiveAvatarView_badgeMarginStart, 0)
            val badgeEndMargin = a.getDimensionPixelSize(R.styleable.ActiveAvatarView_badgeMarginEnd, defaultMargin)
            val badgeHeight = a.getDimensionPixelSize(R.styleable.ActiveAvatarView_badgeHeight, defaultHeight)
            val badgeWidth = a.getDimensionPixelSize(R.styleable.ActiveAvatarView_badgeHeight, defaultWidth)

            with(activeBadge.layoutParams as LayoutParams) {
                width = badgeWidth
                height = badgeHeight
                setMargins(badgeStartMargin, badgeTopMargin, badgeEndMargin, badgeBottomMargin)
            }

            isActive = a.getBoolean(R.styleable.ActiveAvatarView_isActive, false)
        } catch (e: Exception) {
            Timber.wtf(e)
        } finally {
            a.recycle()
        }
    }


    fun setImageURI(uri: Uri?) = this.avatar.setImageURI(uri)

    fun setImageURI(uri: String?) = this.avatar.setImageURI(uri)

    fun setImageDrawable(drawable: Drawable?) = this.avatar.setImageDrawable(drawable)

    fun setImageBackground(drawable: Drawable?) {
        this.avatar.background = drawable
    }
}