package com.under9.android.lib.widget

import android.content.Context
import android.content.res.ColorStateList
import android.graphics.drawable.ColorDrawable
import android.graphics.drawable.Drawable
import android.util.AttributeSet
import androidx.annotation.AttrRes
import androidx.appcompat.widget.Toolbar
import androidx.core.content.ContextCompat
import com.under9.android.lib.util.ColorUtils
import com.under9.android.lib.util.ViewUtils
import timber.log.Timber

class AutoColorToolbar : Toolbar {
    private var titleLightColor: ColorStateList? = null
    private var titleDarkColor: ColorStateList? = null
    private var iconLightColor: Int = -1
    private var iconDarkColor: Int = -1
    
    constructor(context: Context): this(context, null)

    constructor(context: Context, attributeSet: AttributeSet?): this(context, attributeSet, androidx.appcompat.R.attr.toolbarStyle)

    constructor(context: Context, attributeSet: AttributeSet?, @AttrRes defStyleAttr: Int): super(context, attributeSet, defStyleAttr) {
        val ta = context.obtainStyledAttributes(attributeSet, R.styleable.AutoColorToolbar, defStyleAttr, androidx.appcompat.R.attr.toolbarStyle)

        try {
            if (ta.hasValue(R.styleable.AutoColorToolbar_titleDarkColor)) {
                titleDarkColor = ta.getColorStateList(R.styleable.AutoColorToolbar_titleDarkColor)
            }

            if (ta.hasValue(R.styleable.AutoColorToolbar_titleLightColor)) {
                titleLightColor = ta.getColorStateList(R.styleable.AutoColorToolbar_titleLightColor)
            }

            if (ta.hasValue(R.styleable.AutoColorToolbar_iconDarkColor)) {
                iconDarkColor = ContextCompat.getColor(context, ta.getResourceId(R.styleable.AutoColorToolbar_iconDarkColor, -1))
            }

            if (ta.hasValue(R.styleable.AutoColorToolbar_iconLightColor)) {
                iconLightColor = ContextCompat.getColor(context, ta.getResourceId(R.styleable.AutoColorToolbar_iconLightColor, -1))
            }
        } finally {
            ta.recycle()
        }

        Timber.d("background=$background")
    }

    override fun inflateMenu(resId: Int) {
        super.inflateMenu(resId)
    }

    override fun setBackgroundColor(color: Int) {
        super.setBackgroundColor(color)
        mayRefreshColor()
    }

    override fun setTitle(title: CharSequence?) {
        super.setTitle(title)
        Timber.d("setTitle=$title")

        if (background is ColorDrawable) {
            val isDark = ColorUtils.isDark((background as ColorDrawable).color)

            if (isDark) {
                Timber.d("Dark, dark=$titleDarkColor, icon=$iconDarkColor, light=$titleLightColor, icon=$iconLightColor")
                titleDarkColor?.let { setTitleTextColor(it) }

            } else {
                Timber.d("Not dark, dark=$titleDarkColor, icon=$iconDarkColor, light=$titleLightColor, icon=$iconLightColor")
                titleLightColor?.let { setTitleTextColor(it) }
            }
        }
    }

    override fun setNavigationIcon(icon: Drawable?) {
        if (background is ColorDrawable) {
            val isDark = ColorUtils.isDark((background as ColorDrawable).color)
            val iconColor = if (isDark) {
                iconDarkColor
            } else {
                iconLightColor
            }

            if (icon != null) {
                val d = ColorUtils.getTintedDrawable(icon, iconColor)
                super.setNavigationIcon(d)
            } else {
                super.setNavigationIcon(icon)
            }

        } else {
            super.setNavigationIcon(icon)
        }

    }

    fun mayRefreshColor() {
        if (menu.size() != 0) {
            val isDark = ColorUtils.isDark((background as ColorDrawable).color)

            if (isDark) {
                ViewUtils.updateToolbarColor(menu, iconDarkColor)
            } else {
                ViewUtils.updateToolbarColor(menu, iconLightColor)
            }
        }
    }
}