package com.under9.android.lib.widget

import android.app.Activity
import android.content.Context
import android.graphics.Color
import android.os.Build
import android.view.View
import android.view.Window
import com.under9.android.lib.util.UiUtil

class SystemUIColorRestorer(private val context: Context, private val targetWindow: Window) {

    private val originalSystemUiVisibility: Int
    private val decorView: View
    private val hasLightStatusBar: Boolean
    private val hasLightNavBar: Boolean

    private var originalStatusBarColor: Int? = null
    private var originalNavBarColor: Int? = null

    init {
        with((context as Activity)) {
            originalSystemUiVisibility = targetWindow.decorView.systemUiVisibility
            decorView = targetWindow.decorView
            hasLightStatusBar = originalSystemUiVisibility and View.SYSTEM_UI_FLAG_LIGHT_STATUS_BAR == View.SYSTEM_UI_FLAG_LIGHT_STATUS_BAR
            hasLightNavBar = originalSystemUiVisibility and View.SYSTEM_UI_FLAG_LIGHT_NAVIGATION_BAR == View.SYSTEM_UI_FLAG_LIGHT_NAVIGATION_BAR

            if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP) {

                originalStatusBarColor = targetWindow.statusBarColor
                if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O_MR1) {
                    originalNavBarColor = targetWindow.navigationBarColor
                }
            }
        }
    }

    fun darkenUI() {
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP) {
            targetWindow.statusBarColor = Color.BLACK
            targetWindow.navigationBarColor = Color.BLACK
        }

        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.M) {
            decorView.systemUiVisibility = 0
        }
    }
    fun restoreColor() {
        val activity = context as Activity

        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP) {
            originalStatusBarColor?.let {
                targetWindow.statusBarColor = it
            } ?: run {
                targetWindow.statusBarColor = UiUtil.themeAttributeToColor(R.attr.under9_themeColorPrimaryDark, context, -1)
            }
            if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O_MR1) {
                originalNavBarColor?.let {
                    targetWindow.navigationBarColor = it
                } ?: run {
                    targetWindow.navigationBarColor = UiUtil.themeAttributeToColor(R.attr.under9_themeNavigationBarColor, context, -1)
                }

                if (hasLightNavBar && hasLightStatusBar) {
                    decorView.systemUiVisibility = View.SYSTEM_UI_FLAG_LIGHT_STATUS_BAR or View.SYSTEM_UI_FLAG_LIGHT_NAVIGATION_BAR
                }
            } else if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.M) {
                if (hasLightStatusBar) {
                    decorView.systemUiVisibility = View.SYSTEM_UI_FLAG_LIGHT_STATUS_BAR
                }
            }
        }
    }

}