package com.under9.android.lib.widget

import androidx.lifecycle.Lifecycle
import androidx.lifecycle.LifecycleObserver
import androidx.lifecycle.OnLifecycleEvent
import java.util.*

/**
 * This is used for any views that are adding on top of
 *
 * @see com.ninegag.android.app.ui.HomeActivity
 * @see com.ninegag.android.app.ui.SwipeablePostCommentsActivity
 * @see com.ninegag.android.app.ui.PostCommentsActivity
 * @see com.ninegag.android.app.ui.MultiPageSectionListActivity
 * @see com.ninegag.android.app.ui.SearchActivity
 */
class ViewStack: LifecycleObserver {
    private val stack = Stack<StackableView>()

    fun push(stackableView: StackableView) {
        stack.push(stackableView)
    }

    fun peek(): StackableView? {
        if (stack.size == 0) return null
        return stack.peek()
    }

    fun onBackPressed(): Boolean {
        if (stack.size == 0) return false
        val stackableView = stack.pop()
        stackableView.dismiss()
        return true
    }

    /**
     * Just an interface when view is dismissed
     */
    interface StackableView {
        fun dismiss()
    }

    interface StackableHolder {
        fun pushViewStack(stackableView: StackableView)
        fun onBackPressed();
    }

    /**
     * Just simply clear and destroy to avoid any memory leak
     * Can consider about how to restore the view stack later in future
     */
    @OnLifecycleEvent(Lifecycle.Event.ON_DESTROY)
    fun destroy() {
        while (stack.size != 0) {
            onBackPressed()
        }
        stack.clear()
    }
}