package com.under9.android.lib.widget.image.frescoprocessor

import android.graphics.*
import com.facebook.cache.common.CacheKey
import com.facebook.cache.common.SimpleCacheKey
import com.facebook.common.references.CloseableReference
import com.facebook.imagepipeline.bitmaps.PlatformBitmapFactory
import com.facebook.imagepipeline.request.BasePostprocessor

/**
 *
 */
class SquareCroppingPostprocessor : BasePostprocessor {
    private val cornerRadius: Int
    private val maximumSize: Int

    constructor() : this(0, -1)

    constructor(cornerRadius: Int) : this(cornerRadius, -1)

    constructor(cornerRadius: Int, maximumSize: Int) {
        this.cornerRadius = cornerRadius
        this.maximumSize = maximumSize
    }

    override fun process(sourceBitmap: Bitmap?, bitmapFactory: PlatformBitmapFactory?): CloseableReference<Bitmap> {

        if (sourceBitmap == null || bitmapFactory == null) {
            return super.process(sourceBitmap, bitmapFactory)
        } else {
            var croppedBitmap: Bitmap? = if (sourceBitmap.width >= sourceBitmap.height) {
                Bitmap.createBitmap(sourceBitmap, sourceBitmap.width / 2 - sourceBitmap.height / 2, 0, sourceBitmap.height, sourceBitmap.height)
            } else {
                Bitmap.createBitmap(sourceBitmap, 0, sourceBitmap.height / 2 - sourceBitmap.width / 2, sourceBitmap.width, sourceBitmap.width)
            }

            croppedBitmap = if (this.maximumSize != -1) {
                Bitmap.createScaledBitmap(croppedBitmap, maximumSize, maximumSize, false)
            } else {
                croppedBitmap
            }

            return if (croppedBitmap != null) {
                val paint = Paint()
                paint.isAntiAlias = true
                val target = Bitmap.createBitmap(croppedBitmap.width, croppedBitmap.height, Bitmap.Config.ARGB_8888)
                val canvas = Canvas(target)
                val rectF = RectF(0f, 0f, croppedBitmap.width.toFloat(), croppedBitmap.height.toFloat())
                canvas.drawRoundRect(rectF, cornerRadius.toFloat(), cornerRadius.toFloat(), paint)
                paint.xfermode = PorterDuffXfermode(PorterDuff.Mode.SRC_IN)
                canvas.drawBitmap(croppedBitmap, 0f, 0f, paint)

                bitmapFactory.createBitmap(target, 0, 0, croppedBitmap.width, croppedBitmap.height)
            } else {
                super.process(sourceBitmap, bitmapFactory)
            }
        }
    }

    override fun getName(): String {
        return javaClass.name
    }

    override fun getPostprocessorCacheKey(): CacheKey? {
        return SimpleCacheKey("radius=$cornerRadius")
    }
}