package com.under9.android.lib.widget.media.overlayv3

import com.under9.android.lib.util.Irrelevant
import com.under9.android.lib.view.BasePresenter
import com.under9.android.lib.view.Presenter
import io.reactivex.Observable
import io.reactivex.disposables.CompositeDisposable
import io.reactivex.disposables.Disposable

class OverlayPresenterV3 : BasePresenter<OverlayPresenterV3.View>() {

    private var disposables: CompositeDisposable? = null

    interface View : Presenter.View {
        fun longClicks(): Observable<Irrelevant>

        fun showMedia()

        fun hideMedia()

        fun darkenSystemUI()
    }

    override fun onViewAttached(view: OverlayPresenterV3.View?) {
        super.onViewAttached(view)
        view!!.darkenSystemUI()
        initDisposables(view)
    }

    private fun initDisposables(view: View) {
        resetDisposables()

        if (disposables == null) {
            disposables = CompositeDisposable()
        }

    }

    override fun onViewDetached() {
        view?.let {
            super.onViewDetached()
            resetDisposables()
        }
    }

    private fun resetDisposables() {
        if (disposables != null && !disposables!!.isDisposed) {
            disposables?.dispose()
            disposables = null
        }
    }

    fun addDisposable(disposable: Disposable) = disposables?.let { it.add(disposable) }

    companion object {
        val SCOPE = "OverlayModule"
    }
}
