package com.under9.android.lib.widget.media.overlayv3

import android.app.Activity
import android.content.Context
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.view.animation.AccelerateInterpolator
import android.view.animation.DecelerateInterpolator
import android.widget.FrameLayout
import androidx.constraintlayout.widget.ConstraintSet
import androidx.constraintlayout.widget.ConstraintSet.CHAIN_PACKED
import androidx.constraintlayout.widget.ConstraintSet.PARENT_ID
import androidx.transition.AutoTransition
import androidx.transition.TransitionManager
import androidx.transition.TransitionSet
import com.facebook.imagepipeline.listener.RequestListener
import com.facebook.imagepipeline.request.ImageRequest
import com.jakewharton.rxbinding2.view.RxView
import com.under9.android.lib.tracker.pageview.ViewTracker
import com.under9.android.lib.util.Irrelevant
import com.under9.android.lib.view.Presenter
import com.under9.android.lib.widget.R
import com.under9.android.lib.widget.SimpleDragLayout
import com.under9.android.lib.widget.SystemUIColorRestorer
import com.under9.android.lib.widget.ViewStack
import com.under9.android.lib.widget.uiv3.UIV
import com.under9.android.lib.widget.uiv3.adapter.UIVAdapter
import com.under9.android.lib.widget.uiv3.controller.MeasurementController
import com.under9.android.lib.widget.uiv3.controller.UIVVideoController
import io.reactivex.Observable
import io.reactivex.disposables.Disposable
import io.reactivex.subjects.PublishSubject
import kotlinx.android.synthetic.main.view_overlay_v4.view.*
import java.util.concurrent.TimeUnit

class OverlayViewV3(context: Context,
                    private val parentView: ViewGroup,
                    var overlayable: IOverlayable,
                    uivAdapter: UIVAdapter,
                    // Can be init in builder or init by setActionCallback
                    private var actionCallback: OverlayActionCallback?,
                    private val viewTracker: ViewTracker? = null,
                    private val videoTracker: ViewTracker? = null,
                    private val actionViews: List<View>) : FrameLayout(context), OverlayPresenterV3.View, ViewStack.StackableView {

    /** Click listeners **/
    private val clickObservable = PublishSubject.create<Irrelevant>()
    private val longClickObservable = PublishSubject.create<Irrelevant>()

    private val onClickListener = com.under9.android.lib.widget.uiv3.UniversalImageView.OnClickListener { _, _, _ ->
        run {
            if (enabledFullScreenMode) {
                disableFullScreenMode()
            } else {
                enableFullScreenMode()
            }
            clickObservable.onNext(Irrelevant.INSTANCE)
        }
    }
    private val onLongClickListener = com.under9.android.lib.widget.uiv3.UniversalImageView.OnLongClickListener { _, _, _ -> longClickObservable.onNext(Irrelevant.INSTANCE) }

    private var presenter: OverlayPresenterV3

    /** System navigation settings **/

    private var openStateCallback: OverlayViewOpenState = {}
    private val systemUIRestorer: SystemUIColorRestorer
    private var enabledFullScreenMode = false
    /** Drag state listeners **/
    private val listener = SimpleDragLayout.Listener {
        if (getContext() is ViewStack.StackableHolder) {
            (getContext() as ViewStack.StackableHolder).onBackPressed()
        } else {
            dismiss()
        }
    }

    private val requestListener: RequestListener by lazy(LazyThreadSafetyMode.NONE) {
        object : RequestListener {
            override fun onRequestStart(request: ImageRequest?, callerContext: Any?, requestId: String, isPrefetch: Boolean) {
                showLoadingScreen()
            }

            override fun onRequestSuccess(request: ImageRequest?, requestId: String, isPrefetch: Boolean) {
                hideLoadingScreen()
            }

            override fun onRequestFailure(request: ImageRequest?, requestId: String, throwable: Throwable?, isPrefetch: Boolean) {
                hideLoadingScreen()
            }

            override fun onRequestCancellation(requestId: String) {
                hideLoadingScreen()
            }

            override fun onProducerStart(requestId: String, producerName: String) {}
            override fun onProducerEvent(requestId: String, producerName: String, eventName: String) {}
            override fun onProducerFinishWithSuccess(requestId: String, producerName: String, extraMap: Map<String, String>?) {}
            override fun onProducerFinishWithFailure(requestId: String, producerName: String, t: Throwable?, extraMap: Map<String, String>?) {}
            override fun onProducerFinishWithCancellation(requestId: String, producerName: String, extraMap: Map<String, String>?) {}
            override fun requiresExtraMap(requestId: String): Boolean {
                return false
            }

            override fun onUltimateProducerReached(requestId: String, producerName: String, successful: Boolean) {}
        }
    }

    init {
        LayoutInflater.from(context).inflate(R.layout.view_overlay_v4, this, true)

        presenter = OverlayPresenterV3()
        parentView.addView(this)
        visibility = View.GONE
        replaceNewContent(uivAdapter)

        draggableUIV.setTransitionEndCallback {
            openStateCallback.invoke(STATE_CLOSE)
            presenter.onViewDetached()
        }
        actionCallback?.let { setActionCallback(it) }
        systemUIRestorer = SystemUIColorRestorer(getContext()!!, (getContext() as Activity).window)
    }

    fun updateOverlayable(overlayable: IOverlayable) {
        this.overlayable = overlayable

    }
    private fun setActionView(actionViews: List<View>) {
        val totalViews = actionViews.size
        for (i in 0 until totalViews) {
            val curView = actionViews[i]
            overlayViewContainer.addView(curView)
            ConstraintSet().apply {
                // set up the basic constrainSet rule which is center and stay bottom
                clone(overlayViewContainer)
                // left to the right of
                connectALeftToRightOfB(a = curView.id, b = PARENT_ID)
                // right to the left of
                connectARightToLeftOfB(a = curView.id, b = PARENT_ID)
                // bottom to the bottom of
                connectABottomToBottomOfB(a = curView.id, b = PARENT_ID)

                setHorizontalChainStyle(curView.id, CHAIN_PACKED)
                setHorizontalBias(curView.id, 0.5F)
                when (totalViews) {
                    // just apply default constrain set
                    1 -> {
                    }
                    2 -> {
                        if (i == 0) {
                            // update right to the left of
                            connectARightToLeftOfB(a = curView.id, b = actionViews[i + 1].id)
                        } else if (i == 1) {
                            // left to the right of
                            connectALeftToRightOfB(a = curView.id, b = actionViews[i - 1].id)
                        }
                    }
                    else -> {
                        //many
                        if (i == 0) {
                            connectARightToLeftOfB(a = curView.id, b = actionViews[i + 1].id)
                            connectALeftToRightOfB(a = curView.id, b = PARENT_ID)
                        } else if (i == totalViews - 1) {
                            connectALeftToRightOfB(a = curView.id, b = actionViews[i - 1].id)
                            connectARightToLeftOfB(a = curView.id, b = PARENT_ID)
                        } else {
                            connectALeftToRightOfB(a = curView.id, b = actionViews[i - 1].id)
                            connectARightToLeftOfB(a = curView.id, b = actionViews[i + 1].id)
                        }
                    }
                }
            }.also {
                it.applyTo(overlayViewContainer)
            }
        }
    }

    fun setActionCallback(actionCallback: OverlayActionCallback) {
        this.actionCallback = actionCallback
        actionViews.forEach { view ->
            presenter.addDisposable(RxView.clicks(view).throttleFirst(200, TimeUnit.MILLISECONDS).subscribe {
                this.actionCallback?.invoke(view.id)
            })
        }
        val activity = context as Activity
    }

    fun showContent() {
        setActionView(actionViews)

        openStateCallback.invoke(STATE_OPEN)

        presenter.onViewAttached(this)
        val transition = TransitionSet()
                .addTransition(AutoTransition())

        TransitionManager.beginDelayedTransition(this, transition)

        visibility = View.VISIBLE

        viewTracker?.start()
        videoTracker?.start()

    }

    fun replaceNewContent(uivAdapter: UIVAdapter) {
        val uivAdapterBuilder = UIV.cloneUIVAdapter(uivAdapter, true).setResizeMode(MeasurementController.RESIZE_MODE_FILL)

        with(draggableUIV) {
            setAdapter(uivAdapterBuilder
                    .setOnClickListener(onClickListener)
                    .setOnLongClickListener(onLongClickListener)
                    .setRequestListener(requestListener)
                    .build())
            setListener(listener)
        }
    }

    fun showLoadingScreen() {
        loadingLayout.visibility = View.VISIBLE
    }

    fun hideLoadingScreen() {
        loadingLayout.visibility = View.GONE
    }

    fun setOpenStateCallback(openStateCallback: OverlayViewOpenState) {
        this.openStateCallback = openStateCallback
    }

    fun addDisposable(disposable: Disposable) = presenter.addDisposable(disposable)

    override fun longClicks(): Observable<Irrelevant> = longClickObservable

    override fun showMedia() {
        draggableUIV.uiv?.visibility = View.VISIBLE
    }

    override fun hideMedia() {
        draggableUIV.uiv?.visibility = View.GONE
    }

    private fun enableFullScreenMode() {
        overlayActionContainer.animate().apply {
            duration = 100
            alpha(0f)
            interpolator = AccelerateInterpolator()
            start()
        }
        actionViews.forEach {
            it.animate().apply {
                duration = 100
                alpha(0f)
                interpolator = AccelerateInterpolator()
                start()
            }
        }
        // TODO #2497
//        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP) {
//            // While in sticky immersive mode, if the user swipes from an edge with a system bar, system bars appear but they're semi-transparent
//            // and the system navigation will be disappeared with a certain amount of tiem
//            decorView.systemUiVisibility = View.SYSTEM_UI_FLAG_LAYOUT_STABLE or View.SYSTEM_UI_FLAG_HIDE_NAVIGATION or View.SYSTEM_UI_FLAG_LAYOUT_HIDE_NAVIGATION or View.SYSTEM_UI_FLAG_FULLSCREEN or View.SYSTEM_UI_FLAG_LAYOUT_FULLSCREEN or View. SYSTEM_UI_FLAG_IMMERSIVE_STICKY
//        }
        enabledFullScreenMode = true
    }

    private fun disableFullScreenMode() {
        overlayActionContainer.animate().apply {
            duration = 100
            alpha(1f)
            interpolator = DecelerateInterpolator()
            start()
        }
        actionViews.forEach {
            it.animate().apply {
                duration = 100
                alpha(1f)
                interpolator = DecelerateInterpolator()
                start()
            }
        }
        // TODO #2497
//        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP) {
//            decorView.systemUiVisibility = View.SYSTEM_UI_FLAG_LAYOUT_STABLE or View.SYSTEM_UI_FLAG_LAYOUT_HIDE_NAVIGATION or View.SYSTEM_UI_FLAG_LAYOUT_FULLSCREEN
//            darkenSystemUI()
//        }

        enabledFullScreenMode = false
    }



    override fun darkenSystemUI() {
        systemUIRestorer.darkenUI()
    }

    override fun <V : Presenter.View> setPresenter(presenter: Presenter<V>) {
        // Not applicable
    }

    override fun dismiss() {
        UIVVideoController.stopCurrentPlayerIfAvailable()
        val transition = TransitionSet()
                .addTransition(AutoTransition())
                .addListener(draggableUIV)
        draggableUIV.setTransition(transition)
        TransitionManager.beginDelayedTransition(this, transition)

        viewTracker?.stop()
        videoTracker?.stop()

        visibility = View.GONE
        removeView(this)
        openStateCallback.invoke(STATE_DISMISSING)
        systemUIRestorer.restoreColor()

    }

    class Builder(init: Builder.() -> Unit) {
        private lateinit var context: Context
        private lateinit var parentView: ViewGroup
        private lateinit var overlayable: IOverlayable
        private lateinit var uivAdapter: UIVAdapter
        private var actionCallback: OverlayActionCallback? = null
        private var viewTracker: ViewTracker? = null
        private var videoTracker: ViewTracker? = null
        private lateinit var actionViews: List<View>

        init {
            init()
        }

        fun context(context: Context) = apply { this.context = context }

        fun attachTo(viewGroup: ViewGroup) = apply { parentView = viewGroup }

        fun overlayable(overlayable: IOverlayable) = apply { this.overlayable = overlayable }

        fun displayContent(uivAdapter: UIVAdapter) = apply { this.uivAdapter = uivAdapter }

        fun actionCallback(actionCallback: OverlayActionCallback) = apply { this.actionCallback = actionCallback }

        fun setVideoTracker(videoTracker: ViewTracker) = apply { this.videoTracker = videoTracker }

        fun setViewTracker(viewTracker: ViewTracker) = apply { this.viewTracker = viewTracker }

        fun setActionViews(actionViews: List<View>) = apply { this.actionViews = actionViews }

        fun build(init: Builder.() -> Unit) = Builder(init).build()

        fun build(): OverlayViewV3 = OverlayViewV3(
                context = context,
                parentView = parentView,
                overlayable = overlayable,
                uivAdapter = uivAdapter,
                actionCallback = actionCallback,
                actionViews = actionViews,
                viewTracker = viewTracker,
                videoTracker = videoTracker
        )
    }

    companion object {
        private val TAG = "OverlayViewV2"
        @JvmField
        val STATE_OPEN = 1
        @JvmField
        val STATE_CLOSE = 2
        @JvmField
        val STATE_DISMISSING = 3
        val SCOPE = "OverlayModule"
    }
}

typealias OverlayActionCallback = (Int) -> Unit
typealias OverlayViewOpenState = (Int) -> Unit