package com.under9.android.theme

import android.content.res.ColorStateList
import android.content.res.Resources
import androidx.annotation.ColorRes
import androidx.annotation.StyleRes
import androidx.collection.SparseArrayCompat
import androidx.core.util.Pair
import androidx.lifecycle.LiveData
import androidx.lifecycle.MutableLiveData
import com.ninegag.android.gagtheme.R
import com.under9.android.lib.util.ColorUtils
import java.lang.IllegalArgumentException

class ThemeStore constructor(
        private val theme: Resources.Theme,
        @StyleRes val themeAttr: Int
) {
    
    val checkedColorStateList: ColorStateList by lazy(LazyThreadSafetyMode.NONE) {
        val accent = attributeArray.get(R.attr.under9_themeColorAccent, -1)
        val iconColor = attributeArray.get(R.attr.under9_themeIconColor, -1)
        ColorUtils.createCheckableStateList(accent, iconColor)
    }

    private val attributeArray: SparseArrayCompat<Int> = SparseArrayCompat()
    val storeUpdateLiveData: LiveData<Pair<ThemeStore, List<Int>>> = MutableLiveData()

    init {
        val attrs = intArrayOf(
                R.attr.under9_themeWindowBackground,            // 0
                R.attr.under9_themeBackground,                  // 1
                R.attr.under9_themeForeground,                  // 2
                R.attr.under9_themeNavigationBarColor,          // 3
                R.attr.under9_themeColorPrimary,                // 4
                R.attr.under9_themeColorPrimaryDark,            // 5
                R.attr.under9_themeColorAccent,                 // 6
                R.attr.under9_themeColorButtonNormal,           // 7
                R.attr.under9_themeColorControlActivated,       // 8
                R.attr.under9_themeColorControlHighlight,       // 9
                R.attr.under9_themeTextColorPrimary,            // 10
                R.attr.under9_themeTextColorPrimaryInverse,     // 11
                R.attr.under9_themeTextColorSecondary,          // 12
                R.attr.under9_themeTextColorSecondaryInverse,   // 13
                R.attr.under9_themeLineColor,                   // 14
                R.attr.under9_themeIconColor,                   // 15
                R.attr.under9_themeActionBarIconColor,          // 16
                R.attr.under9_themeActionBarOverlay,            // 17
                R.attr.under9_themeTabIndicatorColor,           // 18
                R.attr.under9_themePopupOverlay,                // 19
                R.attr.under9_themeButtonOverlay,               // 20

                R.attr.under9_themeLightNavigationBar,          // 21
                R.attr.under9_themeLightStatusBar,              // 22

                R.attr.under9_themeStatusBarColor,              // 23
                R.attr.under9_themeOverlayLightTextColor,       // 24
                R.attr.under9_themeOverlayLightIconColor,       // 25
                R.attr.under9_themeOverlayDarkTextColor,        // 26
                R.attr.under9_themeOverlayDarkIconColor         // 27
        )
        val ta = theme.obtainStyledAttributes(
                themeAttr, attrs
        )

        with(ta) {
            for (i in 0 until attrs.size) {
                when (val attrId = attrs[i]) {
                    R.attr.under9_themeActionBarOverlay,
                    R.attr.under9_themePopupOverlay,
                    R.attr.under9_themeButtonOverlay -> attributeArray.put(attrId, getResourceId(i, -1))

                    R.attr.under9_themeLightNavigationBar,
                    R.attr.under9_themeLightStatusBar -> {
                        val isTrue = getBoolean(i, false)
                        attributeArray.put(attrId, if (isTrue) 1 else 0)
                    }

                    else -> attributeArray.put(attrId, getColor(i, -1))
                }
            }
        }



        ta.recycle()
    }

    /**
     *
     */
    @Throws
    fun updateColor(ids: IntArray, colors: IntArray) {
        if (ids.size != colors.size) throw IllegalArgumentException("Number of IDs must match the number of colors")

        val hasUpdated: MutableList<Int> = arrayListOf()
        for (i in 0 until ids.size) {
            if (attributeArray.get(ids[i]) != colors[i]) {
                attributeArray.put(ids[i], colors[i])
                hasUpdated.add(ids[i])
            }
        }

        if (hasUpdated.isNotEmpty()) {
            (storeUpdateLiveData as MutableLiveData).postValue(Pair.create(this, hasUpdated))
        }
    }

    fun getColor(attr: Int): Int = attributeArray.get(attr, -1)
}